<?php

namespace  xenMade\LD\ControllerPlugin;

use XF\ControllerPlugin\AbstractPlugin;

class Link extends AbstractPlugin
{
	public function getCategoryListData(\xenMade\LD\Entity\Category $category = null)
	{
		$categoryRepo = $this->getCategoryRepo();
		$categories = $categoryRepo->getViewableCategories();
		$categoryTree = $categoryRepo->createCategoryTree($categories);
		$categoryExtras = $categoryRepo->getCategoryListExtras($categoryTree);

		return [
			'categories' => $categories,
			'categoryTree' => $categoryTree,
			'categoryExtras' => $categoryExtras
		];
	}

	public function getCoreListData(array $sourceCategoryIds, \xenMade\LD\Entity\Category $category = null)
	{
		$linkRepo = $this->getLinkRepo();

		$allowOwnPending = is_callable([$this->controller, 'hasContentPendingApproval'])
			? $this->controller->hasContentPendingApproval()
			: true;

		$linkFinder = $linkRepo->findLinksForLinkList($sourceCategoryIds, [
			'allowOwnPending' => $allowOwnPending
		]);

		$filters = $this->getLinkFilterInput();
		$this->applyLinkFilters($linkFinder, $filters);

		$totalLinks = $linkFinder->total();

		$page = $this->filterPage();
		$perPage = $this->options()->ldLinksPerPage;

		$linkFinder->limitByPage($page, $perPage);
		$links = $linkFinder->fetch()->filterViewable();

		if (!empty($filters['creator_id']))
		{
			$creatorFilter = $this->em()->find('XF:User', $filters['creator_id']);
		}
		else
		{
			$creatorFilter = null;
		}

		$canInlineMod = false;
		foreach ($links AS $link)
		{
			/** @var \xenMade\LD\Entity\Link $link */
			if ($link->canUseInlineModeration())
			{
				$canInlineMod = true;
				break;
			}
		}

		return [
			'links' => $links,
			'filters' => $filters,
			'creatorFilter' => $creatorFilter,
			'canInlineMod' => $canInlineMod,

			'total' => $totalLinks,
			'page' => $page,
			'perPage' => $perPage,
		];
	}

	public function applyLinkFilters(\xenMade\LD\Finder\Link $linkFinder, array $filters)
	{
		if (!empty($filters['prefix_id']))
		{
			$linkFinder->where('prefix_id', intval($filters['prefix_id']));
		}

		if (!empty($filters['type']))
		{
			switch ($filters['type'])
			{
				case 'free':
					$linkFinder->where('price', 0);
					break;

				case 'paid':
					$linkFinder->where('price', '>', 0);
					break;
			}
		}

		if (!empty($filters['creator_id']))
		{
			$linkFinder->where('user_id', intval($filters['creator_id']));
		}

		$sorts = $this->getAvailableLinkSorts();

		if (!empty($filters['order']) && isset($sorts[$filters['order']]))
		{
			$linkFinder->order($sorts[$filters['order']], $filters['direction']);
		}
	}

	public function getLinkFilterInput()
	{
		$filters = [];

		$input = $this->filter([
			'prefix_id' => 'uint',
			'type' => 'str',
			'creator' => 'str',
			'creator_id' => 'uint',
			'order' => 'str',
			'direction' => 'str'
		]);

		if ($input['prefix_id'])
		{
			$filters['prefix_id'] = $input['prefix_id'];
		}

		if ($input['type'] && ($input['type'] == 'free' || $input['type'] == 'paid'))
		{
			$filters['type'] = $input['type'];
		}

		if ($input['creator_id'])
		{
			$filters['creator_id'] = $input['creator_id'];
		}
		else if ($input['creator'])
		{
			$user = $this->em()->findOne('XF:User', ['username' => $input['creator']]);
			if ($user)
			{
				$filters['creator_id'] = $user->user_id;
			}
		}

		$sorts = $this->getAvailableLinkSorts();

		if ($input['order'] && isset($sorts[$input['order']]))
		{
			if (!in_array($input['direction'], ['asc', 'desc']))
			{
				$input['direction'] = 'desc';
			}

			$defaultOrder = $this->options()->ldListDefaultOrder ?: 'insert_date';
			$defaultDir = $defaultOrder == 'title' ? 'asc' : 'desc';

			if ($input['order'] != $defaultOrder || $input['direction'] != $defaultDir)
			{
				$filters['order'] = $input['order'];
				$filters['direction'] = $input['direction'];
			}
		}

		return $filters;
	}

	public function getAvailableLinkSorts()
	{
		return [
			'insert_date' => 'insert_date',
			'rating_weighted' => 'rating_weighted',
			'click_count' => 'click_count',
			'title' => 'title'
		];
	}

	public function actionFilters(\xenMade\LD\Entity\Category $category = null)
	{
		$filters = $this->getLinkFilterInput();

		if ($this->filter('apply', 'bool'))
		{
			return $this->redirect($this->buildLink(
				$category ? 'link-directory/categories' : 'link-directory',
				$category,
				$filters
			));
		}

		if (!empty($filters['creator_id']))
		{
			$creatorFilter = $this->em()->find('XF:User', $filters['creator_id']);
		}
		else
		{
			$creatorFilter = null;
		}

		$applicableCategories = $this->getCategoryRepo()->getViewableCategories($category);
		$applicableCategoryIds = $applicableCategories->keys();
		if ($category)
		{
			$applicableCategoryIds[] = $category->category_id;
		}

		$availablePrefixIds = $this->repository('xenMade\LD:CategoryPrefix')->getPrefixIdsInContent($applicableCategoryIds);
		$prefixes = $this->repository('xenMade\LD:LinkPrefix')->findPrefixesForList()
			->where('prefix_id', $availablePrefixIds)
			->fetch();

		$showTypeFilters = false;


		$defaultOrder = $this->options()->ldListDefaultOrder ?: 'insert_date';
		$defaultDir = $defaultOrder == 'title' ? 'asc' : 'desc';

		if (empty($filters['order']))
		{
			$filters['order'] = $defaultOrder;
		}
		if (empty($filters['direction']))
		{
			$filters['direction'] = $defaultDir;
		}

		$viewParams = [
			'category' => $category,
			'prefixesGrouped' => $prefixes->groupBy('prefix_group_id'),
			'filters' => $filters,
			'creatorFilter' => $creatorFilter,
			'showTypeFilters' => $showTypeFilters
		];
		return $this->view('xenMade\LD:Filters', 'ld_filters', $viewParams);
	}

	/**
	 * @return \xenMade\LD\Repository\Link
	 */
	protected function getLinkRepo()
	{
		return $this->repository('xenMade\LD:Link');
	}

	/**
	 * @return \xenMade\LD\Repository\Category
	 */
	protected function getCategoryRepo()
	{
		return $this->repository('xenMade\LD:Category');
	}
}