<?php

namespace xenMade\LD\Entity;

use XF\Entity\AbstractCategoryTree;
use XF\Mvc\Entity\Structure;

class Category extends AbstractCategoryTree
{
    protected $_viewableDescendants = [];

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xenmade_ld_category';
        $structure->shortName = 'xenMade\LD:Category';
        $structure->primaryKey = 'category_id';
        $structure->contentType = 'ld_category';

        $structure->columns = [
            'category_id' => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'title' => ['type' => self::STR, 'maxLength' => 100, 'required' => 'please_enter_valid_title'],
            'description' => ['type' => self::STR, 'default' => ''],
            'link_count' => ['type' => self::UINT, 'default' => 0, 'forced' => true],
            'last_update' => ['type' => self::UINT, 'default' => 0],
            'last_link_title' => ['type' => self::STR, 'default' => '', 'maxLength' => 100, 'censor' => true],
            'last_link_id' => ['type' => self::UINT, 'default' => 0],
            'field_cache' => ['type' => self::SERIALIZED_ARRAY, 'default' => []],
            'prefix_cache' => ['type' => self::SERIALIZED_ARRAY, 'default' => []],
            'require_prefix' => ['type' => self::BOOL, 'default' => false],
            'allow_link' => ['type' => self::BOOL, 'default' => true],
            'always_moderate_create' => ['type' => self::BOOL, 'default' => false],
            'always_moderate_update' => ['type' => self::BOOL, 'default' => false],
            'backlink' => ['type' => self::STR, 'default' => ''],
            'backlink_location' => ['type' => self::BOOL, 'default' => false],
            'min_tags' => ['type' => self::UINT, 'forced' => true, 'default' => 0, 'max' => 100],
            'thread_node_id' => ['type' => self::UINT, 'default' => 0],
            'thread_prefix_id' => ['type' => self::UINT, 'default' => 0],
        ];

        $structure->getters = [
            'prefixes' => true
        ];

        $structure->relations = [
            'Watch' => [
                'entity' => 'xenMade\LD:CategoryWatch',
                'type' => self::TO_MANY,
                'conditions' => 'category_id',
                'key' => 'user_id'
            ],
            'ThreadForum' => [
                'entity' => 'XF:Forum',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['node_id', '=', '$thread_node_id']
                ],
                'primary' => true,
                'with' => 'Node'
            ]
        ];

        $structure->options = [
            'delete_links' => true
        ];

        static::addCategoryTreeStructureElements($structure);

        return $structure;
    }


    public function getCategoryListExtras()
    {
        return [
            'link_count' => $this->link_count,
            'last_update' => $this->last_update,
            'last_link_title' => $this->last_link_title,
            'last_link_id' => $this->last_link_id
        ];
    }

    public function cacheViewableDescendents(array $descendents, $userId = null)
    {
        if ($userId === null)
        {
            $userId = \XF::visitor()->user_id;
        }

        $this->_viewableDescendants[$userId] = $descendents;
    }

    public function getNewLink()
    {
        $link = $this->_em->create('xenMade\LD:Link');
        $link->category_id = $this->category_id;

        return $link;
    }

    public function getBreadcrumbs($includeSelf = true, $linkType = 'public')
    {
        return $this->_getBreadcrumbs($includeSelf, $linkType, 'link-directory/categories');
    }

    public function getNewContentState(\xenMade\LD\Entity\Link $link = null)
    {
        $visitor = \XF::visitor();

        if ($visitor->user_id && $this->hasPermission('approveUnapprove'))
        {
            return 'visible';
        }

        if (!$visitor->hasPermission('general', 'submitWithoutApproval'))
        {
            return 'moderated';
        }

        if ($link)
        {
            return $this->always_moderate_update ? 'moderated' : 'visible';
        }
        else
        {
            return $this->always_moderate_create ? 'moderated' : 'visible';
        }
    }

    public function getPrefixes()
    {
        if (!$this->prefix_cache)
        {
            return $this->_em->getEmptyCollection();
        }

        $prefixes = $this->finder('xenMade\LD:LinkPrefix')
            ->where('prefix_id', $this->prefix_cache)
            ->order('materialized_order')
            ->fetch();

        return $prefixes;
    }

    public function getUsablePrefixes($forcePrefix = null)
    {
        $prefixes = $this->prefixes;

        if ($forcePrefix instanceof \xenMade\LD\Entity\LinkPrefix)
        {
            $forcePrefix = $forcePrefix->prefix_id;
        }

        $prefixes = $prefixes->filter(function($prefix) use ($forcePrefix)
        {
            if ($forcePrefix && $forcePrefix == $prefix->prefix_id)
            {
                return true;
            }
            return $this->isPrefixUsable($prefix);
        });

        return $prefixes->groupBy('prefix_group_id');
    }

    public function isPrefixUsable($prefix, \XF\Entity\User $user = null)
    {
        if (!$this->isPrefixValid($prefix))
        {
            return false;
        }

        if (!($prefix instanceof \xenMade\LD\Entity\LinkPrefix))
        {
            $prefix = $this->em()->find('xenMade\LD:LinkPrefix', $prefix);
            if (!$prefix)
            {
                return false;
            }
        }

        return $prefix->isUsableByUser($user);
    }

    public function isPrefixValid($prefix)
    {
        if ($prefix instanceof \xenMade\LD\Entity\LinkPrefix)
        {
            $prefix = $prefix->prefix_id;
        }

        return (!$prefix || isset($this->prefix_cache[$prefix]));
    }

    public function linkAdded(\xenMade\LD\Entity\Link $link)
    {
        $this->link_count++;

        if ($link->last_update >= $this->last_update)
        {
            $this->last_update = $link->last_update;
            $this->last_link_title = $link->title;
            $this->last_link_id = $link->link_id;
        }
    }

    public function linkRemoved(\xenMade\LD\Entity\Link $link)
    {
        $this->link_count--;

        if ($link->last_update == $this->last_update)
        {
            $this->rebuildLastLink();
        }
    }

    public function dataChanged(\xenMade\LD\Entity\Link $link)
    {
        if ($link->isChanged(['last_update', 'title']))
        {
            if ($link->last_update >= $this->last_update)
            {
                $this->last_update = $link->last_update;
                $this->last_link_title = $link->title;
                $this->last_link_id = $link->link_id;
            }
            else if ($link->getExistingValue('last_update') == $this->last_update)
            {
                $this->rebuildLastLink();
            }
        }
    }

    public function rebuildLastLink()
    {
        $link = $this->db()->fetchRow("
			SELECT *
			FROM xf_xenmade_ld_link
			WHERE category_id = ?
				AND state = 'visible'
			ORDER BY last_update DESC
			LIMIT 1
		", $this->category_id);

        if ($link)
        {
            $this->last_update = $link['last_update'];
            $this->last_link_title = $link['title'];
            $this->last_link_id = $link['link_id'];
        }
        else
        {
            $this->last_update = 0;
            $this->last_link_title = '';
            $this->last_link_id = 0;
        }
    }

    public function rebuildCounters()
    {
        $data = $this->db()->fetchRow("
			SELECT COUNT(*) AS link_count
			FROM xf_xenmade_ld_link
			WHERE category_id = ?
				AND state = 'visible'
		", $this->category_id);

        $this->link_count = $data['link_count'];

        $this->rebuildLastLink();
    }

    public function hasPermission($permission)
    {
        /** @var \xenMade\LD\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        return $visitor->hasLinkCategoryPermission($this->category_id, $permission);
    }

    public function canView(&$error = null)
    {
        return $this->hasPermission('view');
    }

    public function canWatch(&$error = null)
    {
        return (\XF::visitor()->user_id ? true : false);
    }

    public function canAddLink(&$error = null)
    {
        if(!$this->allow_link)
        {
            return false;
        }

        if (!\XF::visitor()->user_id || !$this->hasPermission('add'))
        {
            $error = \XF::phraseDeferred('ld_category_not_allow_new_links');
            return false;
        }

        return true;
    }

    public function canEditTags(\xenMade\LD\Entity\Link $link = null, &$error = null)
    {
        if (!$this->app()->options()->enableTagging)
        {
            return false;
        }

        if (!$link || $link->user_id == \XF::visitor()->user_id)
        {
            if ($this->hasPermission('tagOwnLink'))
            {
                return true;
            }
        }

        return ($this->hasPermission('tagAnyLink') || $this->hasPermission('manageAnyTag'));
    }

    public function getViewableDescendants()
    {
        $userId = \XF::visitor()->user_id;

        if (!isset($this->_viewableDescendants[$userId]))
        {
            $viewable = $this->repository('xenMade\LD:Category')->getViewableCategories($this);
            $this->_viewableDescendants[$userId] = $viewable->toArray();
        }

        return $this->_viewableDescendants[$userId];
    }


	protected function _preSave()
	{

	}

	protected function _postDelete()
	{
        $db = $this->db();

        $db->delete('xf_xenmade_ld_category_field', 'category_id = ?', $this->category_id);
        $db->delete('xf_xenmade_ld_category_prefix', 'category_id = ?', $this->category_id);
        $db->delete('xf_xenmade_ld_category_watch', 'category_id = ?', $this->category_id);

        if ($this->getOption('delete_links'))
        {
            $this->app()->jobManager()->enqueueUnique('ld_link_category_delete' . $this->category_id, 'xenMade\LD:CategoryDelete', ['category_id' => $this->category_id]);
        }
	}
}