<?php

namespace xenMade\LD\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class LinkComment extends Entity
{
    protected $Privacy = null;
    protected $is_banned = null;
    protected $link = null;

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xenmade_ld_link_comment';
        $structure->shortName = 'xenMade\LD:LinkComment';
        $structure->contentType = 'ld_link_comment';
        $structure->primaryKey = 'comment_id';

        $structure->columns = [
            'comment_id' => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'link_id' => ['type' => self::UINT, 'required' => true],
            'user_id' => ['type' => self::UINT, 'required' => true],
            'username' => ['type' => self::STR, 'maxLength' => 50,
                'required' => 'please_enter_valid_name'
            ],
            'post_date' => ['type' => self::UINT, 'required' => true, 'default' => \XF::$time],
            'message' => ['type' => self::STR,
                'required' => 'please_enter_valid_message'
            ],
            'ip_id' => ['type' => self::UINT, 'default' => 0],
            'message_state' => ['type' => self::STR, 'default' => 'visible',
                'allowedValues' => ['visible', 'moderated', 'deleted']
            ],
            'likes' => ['type' => self::UINT, 'forced' => true, 'default' => 0],
            'like_users' => ['type' => self::SERIALIZED_ARRAY, 'default' => []],
            'warning_id' => ['type' => self::UINT, 'default' => 0],
            'warning_message' => ['type' => self::STR, 'default' => '']
        ];
        $structure->behaviors = [
            'XF:Likeable' => ['stateField' => 'message_state'],
            'XF:Indexable' => [
                'checkForUpdates' => ['message', 'link_id', 'user_id', 'post_date', 'message_state']
            ],
            'XF:NewsFeedPublishable' => [
                'usernameField' => 'username',
                'dateField' => 'post_date'
            ]
        ];

        $structure->getters = [
            'link_title' => true
        ];

        $structure->relations = [
            'Link' => [
                'entity' => 'xenMade\LD:Link',
                'type' => self::TO_ONE,
                'conditions' => [['link_id', '=', '$link_id']],
                'primary' => true
            ],
            'User' => [
                'entity' => 'XF:User',
                'type' => self::TO_ONE,
                'conditions' => 'user_id',
                'primary' => true
            ],
            'Likes' => [
                'entity' => 'XF:LikedContent',
                'type' => self::TO_MANY,
                'conditions' => [
                    ['content_type', '=', 'ld_link_comment'],
                    ['content_id', '=', '$link_id']
                ],
                'key' => 'like_user_id',
                'order' => 'like_date'
            ],
            'DeletionLog' => [
                'entity' => 'XF:DeletionLog',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['content_type', '=', 'ld_link_comment'],
                    ['content_id', '=', '$link_id']
                ],
                'primary' => true
            ],
            'ApprovalQueue' => [
                'entity' => 'XF:ApprovalQueue',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['content_type', '=', 'ld_link_comment'],
                    ['content_id', '=', '$link_id']
                ],
                'primary' => true
            ]
        ];

        $structure->options = [
            'log_moderator' => true
        ];

        $structure->defaultWith = [
            'Link'
        ];

        return $structure;
    }


    public function getLinkTitle()
    {
        return $this->Link ? $this->Link->title : '';
    }

	public function canView(&$error = null)
	{
		$visitor = \XF::visitor();

		if (!$this->Link)
		{
			return false;
		}

		if (!$this->canViewCommentsOnLink())
		{
			return false;
		}

		if ($this->message_state == 'moderated')
		{
			if (
				!$visitor->hasPermission('ld_link_comment', 'viewModerated')
				&& (!$visitor->user_id || $visitor->user_id != $this->user_id)
			)
			{
				$error = \XF::phraseDeferred('ld_requested_link_comment_not_found');
				return false;
			}
		}
		else if ($this->message_state == 'deleted')
		{
			if (!$visitor->hasPermission('ld_link_comment', 'viewDeleted'))
			{
				$error = \XF::phraseDeferred('ld_requested_link_comment_not_found');
				return false;
			}
		}

		return true;
	}

    public function canViewCommentsOnLink(&$error = null)
    {
        return $this->canViewFullLinkComment() && \XF::visitor()->hasPermission('ld_link_comment', 'view');
    }

    public function canViewFullLinkComment(&$error = null)
    {
        $visitor = \XF::visitor();
        if ($visitor->user_id == $this->user_id)
        {
            return true;
        }

        if (!$visitor->hasPermission('general', 'viewProfile'))
        {
            return false;
        }

        if (!$this->isPrivacyCheckMet('allow_view_profile', $visitor))
        {
            $error = \XF::phraseDeferred('member_limits_viewing_profile');
            return false;
        }

        if (
            ($this->User->user_state == 'moderated' || $this->User->user_state == 'email_confirm' || $this->User->user_state == 'rejected') &&
            !$visitor->canBypassUserPrivacy()
        )
        {
            $error = \XF::phraseDeferred('this_users_profile_is_not_available');
            return false;
        }

        if ($this->is_banned)
        {
            /** @var UserBan|null $ban */
            $ban = $this->Ban;
            if ($ban && !$ban->end_date && !$visitor->canBypassUserPrivacy())
            {
                $error = \XF::phraseDeferred('this_users_profile_is_not_available');
                return false;
            }
        }

        return true;
    }

    public function isPrivacyCheckMet($privacyKey, \XF\Entity\User $user)
    {
        if (!$this->Privacy)
        {
            return true;
        }

        $privacy = $this->Privacy;
        return $privacy->isPrivacyCheckMet($privacyKey, $user);
    }

    public function canUseInlineModeration(&$error = null)
	{
		$visitor = \XF::visitor();
		return ($visitor->user_id && $visitor->hasPermission('ld_link_comment', 'inlineMod'));
	}

	public function canEdit(&$error = null)
	{
		$visitor = \XF::visitor();

		if (!$visitor->user_id)
		{
			return false;
		}

		if ($visitor->user_id == $this->user_id)
		{
			return $visitor->hasPermission('ld_link_comment', 'editOwn');
		}
		else
		{
			return $visitor->hasPermission('ld_link_comment', 'editAny');
		}
	}

	public function canDelete($type = 'soft', &$error = null)
	{
		$visitor = \XF::visitor();
		if (!$visitor->user_id)
		{
			return false;
		}

		if ($type != 'soft' && !$visitor->hasPermission('ld_link_comment', 'hardDeleteAny'))
		{
			return false;
		}

		if ($visitor->hasPermission('ld_link_comment', 'deleteAny'))
		{
			return true;
		}

		return (	$visitor->user_id == $this->user_id && $visitor->hasPermission('ld_link_comment', 'deleteOwn'));
	}

	public function canUndelete(&$error = null)
	{
		$visitor = \XF::visitor();

		return ($visitor->user_id && $visitor->hasPermission('ld_link_comment', 'undelete'));
	}

	public function canApproveUnapprove(&$error = null)
	{
		$visitor = \XF::visitor();
		return ($visitor->user_id && $visitor->hasPermission('ld_link_comment', 'approveUnapprove'));
	}

	public function canWarn(&$error = null)
	{
		$visitor = \XF::visitor();

		if ($this->warning_id
			|| !$this->user_id
			|| !$visitor->user_id
			|| $this->user_id == $visitor->user_id
			|| !$visitor->hasPermission('ld_link_comment', 'warn')
		)
		{
			return false;
		}

		return ($this->User && $this->User->isWarnable());
	}

	public function canReport(&$error = null, \XF\Entity\User $asUser = null)
	{
		$asUser = $asUser ?: \XF::visitor();
		return $asUser->canReport($error);
	}

	public function canLike(&$error = null)
	{
		$visitor = \XF::visitor();
		if (!$visitor->user_id)
		{
			return false;
		}

		if ($this->message_state != 'visible')
		{
			return false;
		}

		if ($this->user_id == $visitor->user_id)
		{
			$error = \XF::phraseDeferred('liking_own_content_cheating');
			return false;
		}

		return $visitor->hasPermission('ld_link_comment', 'like');
	}

	public function canSendModeratorActionAlert()
	{
		$visitor = \XF::visitor();

		if (!$visitor->user_id || $visitor->user_id == $this->user_id)
		{
			return false;
		}

		if ($this->message_state != 'visible')
		{
			return false;
		}

		return (
			$visitor->hasPermission('ld_link_comment', 'deleteAny')
			|| $visitor->hasPermission('ld_link_comment', 'editAny')
		);
	}

	public function isVisible()
	{
		return ($this->message_state == 'visible');
	}

	public function isLiked()
	{
		$visitor = \XF::visitor();
		if (!$visitor->user_id)
		{
			return false;
		}

		return isset($this->Likes[$visitor->user_id]);
	}

	public function isIgnored()
	{
		return \XF::visitor()->isIgnoring($this->user_id);
	}

	public function canCleanSpam()
	{
		return (\XF::visitor()->canCleanSpam() && $this->User && $this->User->isPossibleSpammer());
	}

	public function rebuildCounters()
	{
		if (!$this->rebuildFirstCommentInfo())
		{

		}
		else
		{
			$this->rebuildLastCommentInfo();
			$this->rebuildCommentCount();
		}

		$this->rebuildLatestCommentIds();

		return true;
	}


	protected function _postSave()
	{
		$visibilityChange = $this->isStateChanged('message_state', 'visible');
		$approvalChange = $this->isStateChanged('message_state', 'moderated');
		$deletionChange = $this->isStateChanged('message_state', 'deleted');

		if ($this->isUpdate())
		{
			if ($visibilityChange == 'enter')
			{
				if ($approvalChange)
				{
					$this->submitHamData();
				}
			}
			else if ($visibilityChange == 'leave')
			{
				$this->commentHidden();
			}

			if ($deletionChange == 'leave' && $this->DeletionLog)
			{
				$this->DeletionLog->delete();
			}

			if ($approvalChange == 'leave' && $this->ApprovalQueue)
			{
				$this->ApprovalQueue->delete();
			}
		}

		if ($approvalChange == 'enter')
		{
			$approvalQueue = $this->getRelationOrDefault('ApprovalQueue', false);
			$approvalQueue->content_date = $this->post_date;
			$approvalQueue->save();
		}
		else if ($deletionChange == 'enter' && !$this->DeletionLog)
		{
			$delLog = $this->getRelationOrDefault('DeletionLog', false);
			$delLog->setFromVisitor();
			$delLog->save();
		}

		if ($this->isUpdate() && $this->getOption('log_moderator'))
		{
			$this->app()->logger()->logModeratorChanges('ld_link_comment', $this);
		}

        $this->updateLinkRecord();
	}

	protected function commentHidden($hardDelete = false)
	{
		/** @var \XF\Repository\UserAlert $alertRepo */
		$alertRepo = $this->repository('XF:UserAlert');
		$alertRepo->fastDeleteAlertsForContent('ld_link_comment', $this->comment_id);
	}

	protected function submitHamData()
	{
		/** @var \XF\Spam\ContentChecker $submitter */
		$submitter = $this->app()->container('spam.contentHamSubmitter');
		$submitter->submitHam('ld_link_comment', $this->comment_id);
	}

	protected function _postDelete()
	{
		if ($this->message_state == 'visible')
		{
			$this->commentHidden(true);
		}

		if ($this->message_state == 'deleted' && $this->DeletionLog)
		{
			$this->DeletionLog->delete();
		}

		if ($this->message_state == 'moderated' && $this->ApprovalQueue)
		{
			$this->ApprovalQueue->delete();
		}

		if ($this->getOption('log_moderator'))
		{
			$this->app()->logger()->logModeratorAction('ld_link_comment', $this, 'delete_hard');
		}

        if ($this->message_state == 'visible')
        {
            $this->Link->commentRemoved();
            $this->Link->save();
        }
	}

    protected function updateLinkRecord()
    {
        if (!$this->Link)
        {
            return;
        }

        /** @var \xenMade\LD\Entity\Link $link */
        $link = $this->Link;

        if ($this->isUpdate())
        {
            if ($this->message_state == 'visible')
            {
                $link->commentAdded();
                $link->save();
            }

            if ($this->getExistingValue('message_state') == 'visible')
            {
                $link->commentRemoved();
                $link->save();
            }

            return;
        }

        $visibilityChange = $this->isStateChanged('message_state', 'visible');
        if ($visibilityChange == 'enter' && $link)
        {
            $link->commentAdded();
            $link->save();
        }
        else if ($visibilityChange == 'leave' && $link)
        {
            $link->commentRemoved();
            $link->save();
        }
        else if ($this->isUpdate() && $this->state == 'visible')
        {
            $link->save();
        }
    }


	public function softDelete($reason = '', \XF\Entity\User $byUser = null)
	{
		$byUser = $byUser ?: \XF::visitor();

		if ($this->message_state == 'deleted')
		{
			return false;
		}

		$this->message_state = 'deleted';

		/** @var \XF\Entity\DeletionLog $deletionLog */
		$deletionLog = $this->getRelationOrDefault('DeletionLog');
		$deletionLog->setFromUser($byUser);
		$deletionLog->delete_reason = $reason;

		$this->save();

		return true;
	}

    public function softUndelete()
    {
        if ($this->message_state == 'visible')
        {
            return false;
        }

        $this->message_state = 'visible';

        $this->save();

        return true;
    }

	public function getNewContentState()
	{
		$visitor = \XF::visitor();

		if ($visitor->user_id && $visitor->hasPermission('ld_link_comment', 'approveUnapprove'))
		{
			return 'visible';
		}

		if (!$visitor->hasPermission('general', 'submitWithoutApproval'))
		{
			return 'moderated';
		}

		return 'visible';
	}
}