<?php

namespace xenMade\LD\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class LinkRating extends Entity
{
    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xenmade_ld_link_rating';
        $structure->shortName = 'xenMade\LD:LinkRating';
        $structure->primaryKey = 'rating_id';
        $structure->contentType = 'ld_link_rating';

        $structure->columns = [
            'rating_id' => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'link_id' => ['type' => self::UINT, 'required' => true],
            'user_id' => ['type' => self::UINT, 'required' => true],
            'username' => ['type' => self::STR, 'maxLength' => 50,
                'required' => 'please_enter_valid_name'
            ],
            'rating' => ['type' => self::UINT, 'required' => true, 'min' => 1, 'max' => 5],
            'rating_date' => ['type' => self::UINT, 'default' => \XF::$time],
            'message' => ['type' => self::STR, 'default' => ''],
            'author_response' => ['type' => self::STR, 'default' => ''],
            'rating_state' => ['type' => self::STR, 'default' => 'visible', 'allowedValues' => ['visible', 'deleted']],
            'warning_id' => ['type' => self::UINT, 'default' => 0],
        ];

        $structure->getters = [
            'link_title' => true
        ];

        $structure->behaviors = [];

        $structure->relations = [
            'Link' => [
                'entity' => 'xenMade\LD:Link',
                'type' => self::TO_ONE,
                'conditions' => [['link_id', '=', '$link_id']],
                'primary' => true
            ],
            'User' => [
                'entity' => 'XF:User',
                'type' => self::TO_ONE,
                'conditions' => 'user_id',
                'primary' => true
            ],
            'DeletionLog' => [
                'entity' => 'XF:DeletionLog',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['content_type', '=', 'ld_link_rating'],
                    ['content_id', '=', '$rating_id']
                ],
                'primary' => true
            ]
        ];

        $structure->options = [
            'log_moderator' => true
        ];

        $structure->defaultWith = ['Link'];

        return $structure;
    }

    public function getLinkTitle()
    {
        return $this->Link ? $this->Link->title : '';
    }

	public function canView(&$error = null)
	{
		$link = $this->Link;

		if (!$link || !$link->canView($error))
		{
			return false;
		}

		if ($this->rating_state == 'deleted')
		{
			if (!$link->hasPermission('viewDeleted'))
			{
				return false;
			}
		}

		return true;
	}

	public function canDelete($type = 'soft', &$error = null)
	{
		$visitor = \XF::visitor();
		$link = $this->Link;

		if (!$visitor->user_id || !$link)
		{
			return false;
		}

		if ($type != 'soft')
		{
			return (
				$link->hasPermission('hardDeleteAny')
				&& $link->hasPermission('deleteAny')
			);
		}

		if ($this->user_id == $visitor->user_id && !$this->author_response)
		{
			return true;
		}

		return $link->hasPermission('deleteAny');
	}


	public function canUndelete(&$error = null)
	{
		$visitor = \XF::visitor();
		$link = $this->Link;

		if (!$visitor->user_id || !$link)
		{
			return false;
		}

		return $link->hasPermission('undelete');
	}

	public function canReport(&$error = null, \XF\Entity\User $asUser = null)
	{
		$asUser = $asUser ?: \XF::visitor();
		return $asUser->canReport($error);
	}

	public function canWarn(&$error = null)
	{
		$visitor = \XF::visitor();
		$link = $this->Link;

		if ($this->warning_id
			|| !$link
			|| !$visitor->user_id
			|| $this->user_id == $visitor->user_id
			|| !$link->hasPermission('warn')
		)
		{
			return false;
		}

		$user = $this->User;
		return ($user && $user->isWarnable());
	}

    public function canSendModeratorActionAlert()
    {
        $visitor = \XF::visitor();

        if (!$visitor->user_id || $visitor->user_id == $this->user_id)
        {
            return false;
        }

        if ($this->rating_state != 'visible')
        {
            return false;
        }

        return (
            $visitor->hasPermission('ld_link', 'deleteAny')
            || $visitor->hasPermission('ld_link', 'editAny')
        );
    }

	public function isVisible()
	{
		return (
			$this->rating_state == 'visible'
			&& $this->Link
			&& $this->Link->isVisible()
		);
	}

	public function isIgnored()
	{
		return \XF::visitor()->isIgnoring($this->user_id);
	}

	protected function _preSave()
	{
		if (!$this->user_id || !$this->Link)
		{
			throw new \LogicException("Need user and link");
		}
	}

	protected function _postSave()
	{
		$visibilityChange = $this->isStateChanged('rating_state', 'visible');
		$deletionChange = $this->isStateChanged('rating_state', 'deleted');

		if ($this->isUpdate())
		{
			if ($visibilityChange == 'enter')
			{
				$this->ratingMadeVisible();
			}
			else if ($visibilityChange == 'leave')
			{
				$this->ratingHidden();
			}

			if ($deletionChange == 'leave' && $this->DeletionLog)
			{
				$this->DeletionLog->delete();
			}
		}
		else
		{
			if ($this->rating_state == 'visible')
			{
				$this->ratingMadeVisible();
			}
		}

		if ($deletionChange == 'enter' && !$this->DeletionLog)
		{
			$delLog = $this->getRelationOrDefault('DeletionLog', false);
			$delLog->setFromVisitor();
			$delLog->save();
		}

		if ($this->isUpdate() && $this->getOption('log_moderator'))
		{
			$this->app()->logger()->logModeratorChanges('ld_link_rating', $this);
		}
	}

	protected function ratingMadeVisible()
	{
		$link = $this->Link;

		if ($link)
		{
            $link->ratingAdded($this);
            $link->save();
		}

		if ($this->rebuildRatingCounted() && $link)
		{
			$link->rebuildRating();
		}
	}

	protected function ratingHidden($hardDelete = false)
	{
		$link = $this->Link;

		if ($link)
		{
            $link->ratingRemoved($this);
            $link->save();
		}

		if ($link)
		{
			$link->rating_count--;
		}

		if (
			($link->rating_count && $link) &&
			($this->rebuildRatingCounted() || $hardDelete)
		)
		{
			$link->rebuildRating();
		}
		if ($link)
		{
			$link->saveIfChanged();
		}

		/** @var \XF\Repository\UserAlert $alertRepo */
		$alertRepo = $this->repository('XF:UserAlert');
		$alertRepo->fastDeleteAlertsForContent('ld_link_rating', $this->rating_id);
	}

	protected function rebuildRatingCounted()
	{

	}

	protected function _postDelete()
	{
		if ($this->rating_state == 'visible')
		{
			$this->ratingHidden(true);
		}

		if ($this->rating_state == 'deleted' && $this->DeletionLog)
		{
			$this->DeletionLog->delete();
		}

		if ($this->getOption('log_moderator'))
		{
			$this->app()->logger()->logModeratorAction('ld_link_rating', $this, 'delete_hard');
		}
	}

    public function softUndelete()
    {
        if ($this->rating_state == 'visible')
        {
            return false;
        }

        $this->rating_state = 'visible';

        $this->save();

        return true;
    }

	public function softDelete($reason = '', \XF\Entity\User $byUser = null)
	{
		$byUser = $byUser ?: \XF::visitor();

		if ($this->rating_state == 'deleted')
		{
			return false;
		}

		$this->rating_state = 'deleted';

		/** @var \XF\Entity\DeletionLog $deletionLog */
		$deletionLog = $this->getRelationOrDefault('DeletionLog');
		$deletionLog->setFromUser($byUser);
		$deletionLog->delete_reason = $reason;

		$this->save();

		return true;
	}
}