<?php

namespace xenMade\LD\Import\Importer;

use XF\Import\StepState;

use League\Flysystem\Adapter\Local;
use League\Flysystem\Filesystem;

class LD extends AbstractLDImporter
{
	protected $sourceDb;
	
	public static function getListInfo()
	{
		return [
			'target' => 'xenMade Link Directory',
			'source' => 'LD XF1',
			'beta'   => true
		];
	}

	protected function getBaseConfigDefault()
	{
		return [
			'db' => [
				'host' => '',
				'username' => '',
				'password' => '',
				'dbname' => '',
				'port' => 3306
			],
            'data_dir' => '',
            'internal_data_dir' => ''
		];
	}

	public function validateBaseConfig(array &$baseConfig, array &$errors)
	{
		$fullConfig = array_replace_recursive($this->getBaseConfigDefault(), $baseConfig);
		$missingFields = false;
		
		if ($fullConfig['db']['host'])
		{
			$validDbConnection = false;
			
			try
			{
				$db = new \XF\Db\Mysqli\Adapter($fullConfig['db'], false);
				$db->getConnection();
				$validDbConnection = true;
			}
			catch (\XF\Db\Exception $e)
			{
				$errors[] = \XF::phrase('source_database_connection_details_not_correct_x', ['message' => $e->getMessage()]);
			}
			
			if ($validDbConnection)
			{
				try
				{
					$db->fetchOne("SELECT option_value FROM xf_option WHERE option_id = 'currentVersionId'");
				}
				catch (\XF\Db\Exception $e)
				{
					if ($fullConfig['db']['dbname'] === '')
					{
						$errors[] = \XF::phrase('please_enter_database_name');
					}
					else
					{
						$errors[] = \XF::phrase('table_prefix_or_database_name_is_not_correct');
					}
				}
			}
		}
		else
		{
			$missingFields = true;
		}

        if (empty($fullConfig['confirm']))
        {
            $missingFields = true;
        }

        if ($fullConfig['internal_data_dir'])
        {
            $internalData = rtrim($fullConfig['internal_data_dir'], '/\\ ');

            if (!file_exists($internalData) || !is_dir($internalData))
            {
                $errors[] = \XF::phrase('directory_x_not_found_is_not_readable', ['dir' => $internalData]);
            }
            else if (!file_exists("$internalData/install-lock.php"))
            {
                $errors[] = \XF::phrase('directory_x_does_not_contain_expected_contents', ['dir' => $internalData]);
            }

            $baseConfig['internal_data_dir'] = $internalData; // to make sure it takes the format we expect
        }
        else
        {
            $missingFields = true;
        }

        if ($missingFields)
        {
            $errors[] = \XF::phrase('please_complete_required_fields');
        }

        if(!$missingFields && isset($fullConfig['confirm']))
        {
            $dbTarget = $this->db();
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_category`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_category_field`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_category_prefix`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_category_watch`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_link`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_link_comment`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_link_field`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_link_field_value`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_link_prefix`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_link_prefix_group`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_link_rating`;');
            $dbTarget->query('TRUNCATE `xf_xenmade_ld_link_watch`;');

            $dbTarget->query('DELETE FROM xf_phrase WHERE title LIKE \'ld_link_field%\' AND addon_id = \'\';');
            $dbTarget->query('DELETE FROM xf_phrase WHERE title LIKE \'ld_link_prefix%\' AND addon_id = \'\';');
            $dbTarget->query('DELETE FROM xf_phrase WHERE title LIKE \'ld_prefix_group%\' AND addon_id = \'\';');
            $dbTarget->query('DELETE FROM xf_phrase WHERE title LIKE \'ld_prefix_%\' AND addon_id = \'\';');

            $contentTypes = ['ld_link', 'ld_category', 'ld_link_comment', 'ld_link_rating'];
            $contentTypesQuoted = $dbTarget->quote($contentTypes);

            $contentTypeTables = [
                'xf_approval_queue',
                'xf_deletion_log',
                'xf_liked_content',
                'xf_moderator_log',
                'xf_news_feed',
                'xf_report',
                'xf_tag_content',
                'xf_user_alert'
            ];

            foreach ($contentTypeTables AS $table)
            {
                $dbTarget->delete($table, 'content_type IN (' . $contentTypesQuoted . ')');
            }

            \XF\Util\File::deleteAbstractedDirectory('data://ld_thumbnail');
        }

		return $errors ? false : true;
	}

	public function renderBaseConfigOptions(array $vars)
	{
		return $this->app->templater()->renderTemplate('admin:ld_import_config', $vars);
	}

	public function resetDataForRetainIds()
	{
	}

	protected function getStepConfigDefault()
	{
	}

	public function renderStepConfigOptions(array $vars)
	{
		return '';
	}

	public function validateStepConfig(array $steps, array &$stepConfig, array &$errors)
	{
		return true;
	}

	protected function doInitializeSource()
	{
	    if(!empty($this->baseConfig['db']))
        {
            $dbConfig = $this->baseConfig['db'];

            $this->sourceDb = new \XF\Db\Mysqli\Adapter($dbConfig, false);
        }
	}
	
	public function getSteps()
	{
	    $this->doInitializeSource();

        $status = $this->getTable('phc_linkdirectory_field');

        $steps = [];
        if($status)
        {
            $steps = [
                'linkFields' => [
                    'title' => \XF::phrase('ld_custom_link_fields'),
                ]
            ];
        }

		return array_merge($steps, [
            'categories' => [
                'title' => \XF::phrase('categories'),
            ],
            'linkPrefixes' => [
                'title' => \XF::phrase('ld_link_prefixes'),
                'depends' => ['categories']
            ],
			'links' => [
				'title' => \XF::phrase('ld_links'),
                'depends' => ['categories']
			],
            'linksLikes' => [
                'title' => \XF::phrase('likes'),
                'depends' => ['links']
            ],
            'linkRatings' => [
				'title' => \XF::phrase('ld_ratings'),
                'depends' => ['links']
			],
		]);
	}



    // ########################### STEP: Fields ###############################

    public function stepLinkFields(StepState $state, array $stepConfig, $maxTime, $limit = 25)
    {
        $linkFields = $this->sourceDb->fetchAll("
			SELECT field.*,
				ptitle.phrase_text AS title,
				pdesc.phrase_text AS description
			FROM phc_linkdirectory_field AS field
			INNER JOIN xf_phrase AS ptitle ON
				(ptitle.language_id = 0 AND ptitle.title = CONCAT('ld_field_', field.field_id))
			INNER JOIN xf_phrase AS pdesc ON
				(pdesc.language_id = 0 AND pdesc.title = CONCAT('ld_field_', field.field_id))
		");

        $existingFields = $this->db()->fetchPairs("SELECT field_id, field_id FROM xf_xenmade_ld_link_field");

        foreach ($linkFields AS $linkField)
        {
            $oldId = $linkField['field_id'];

            if (empty($existingFields[$oldId]))
            {

                $linkField['display_group'] = $this->decodeFieldDisplayGroup($linkField['display_group']);
                $linkField['match_params'] = $this->decodeFieldMatchParamsGroup($linkField['match_regex']);

                /** @var \xenMade\LD\Import\Data\LinkField $import */
                $import = $this->getHelper()->setupXfCustomFieldImport('xenMade\LD:LinkField', $linkField);
                $import->bulkSet($this->mapKeys($linkField, []));

                $import->save($oldId);
            }

            $state->imported++;
        }

        return $state->complete();
    }

    // ########################### STEP: Category ###############################

    public function stepCategories(StepState $state, array $stepConfig, $maxTime, $limit = 25)
    {
        $timer = new \XF\Timer($maxTime);

        $categories = $this->getCategories($state->startAfter, $state->end, $limit);

        if (!$categories)
        {
            return $state->complete();
        }

        foreach ($categories AS $oldCategoryId => $category)
        {
            $state->startAfter = $oldCategoryId;

            /** @var \xenMade\LD\Import\Data\Category $import */
            $import = $this->newHandler('xenMade\LD:Category');

            $import->bulkSet($this->mapXfKeys($category, [
                'category_id' => 'ld_cat_id',
                'title',
                'description' => 'pagetext',
                'parent_category_id' => 'parent_ld_cat_id',
                'display_order' => 'order',
                'lft',
                'rgt' => 'depth',
                'depth',
                'link_count',
            ]));


            $breadcrumbData = \XF\Util\Php::safeUnserialize($category['breadcrumb']);
            $import->breadcrumb_data = $breadcrumbData ?: [];
            $import->preventRetainIds();


            $status = $this->getTable('phc_linkdirectory_field');

            if($status)
            {
                $fieldCache = [];

                $fields = $this->sourceDb->fetchAllKeyed("
                                SELECT field.*
                                FROM phc_linkdirectory_field AS field
                            ", 'field_id');

                foreach ($fields as $field)
                {
                    $fieldCache[$field['field_id']] = $field['field_id'];
                }

                $import->field_cache = $fieldCache;
            }


            if ($newCategoryId = $import->save($oldCategoryId))
            {
                $state->imported++;
            }

            if ($timer->limitExceeded())
            {
                break;
            }
        }

        return $state->resumeIfNeeded();
    }

    public function getStepEndCategories()
    {
        return $this->sourceDb->fetchOne("SELECT MAX(ld_cat_id) FROM phc_linkdirectory_categories") ?: 0;
    }

    protected function getCategories($startAfter, $end, $limit)
    {
        return $this->sourceDb->fetchAllKeyed("
			SELECT category.*
			FROM phc_linkdirectory_categories AS category
			WHERE category.ld_cat_id > ? AND category.ld_cat_id <= ?
			ORDER BY category.ld_cat_id
			LIMIT {$limit}
		", 'ld_cat_id', [$startAfter, $end]);
    }



    // ########################### STEP: Prefixes ###############################

    public function stepLinkPrefixes(StepState $state)
    {
        $this->typeMap('ld_category');

        $prefixGroups = $this->sourceDb->fetchAllKeyed("
			SELECT tpg.*, p.phrase_text
			FROM phc_linkdirectory_prefix_group AS tpg
			INNER JOIN xf_phrase AS p ON (p.language_id = 0 AND p.title = CONCAT('ld_prefix_group_', tpg.prefix_group_id))
		", 'prefix_group_id');
        $mappedGroupIds = [];


        foreach ($prefixGroups AS $oldGroupId => $group)
        {
            /** @var \xenMade\LD\Import\Data\LinkPrefixGroup $importGroup */
            $importGroup = $this->newHandler('xenMade\LD:LinkPrefixGroup');
            $importGroup->display_order = $group['display_order'];
            $importGroup->setTitle($group['phrase_text']);

            $newGroupId = $importGroup->save($oldGroupId);
            if ($newGroupId)
            {
                $mappedGroupIds[$oldGroupId] = $newGroupId;
            }
        }

        $prefixes = $this->sourceDb->fetchAllKeyed("
			SELECT tp.*, p.phrase_text
			FROM phc_linkdirectory_prefix AS tp
			INNER JOIN xf_phrase AS p ON (p.language_id = 0 AND p.title = CONCAT('ld_prefix_', tp.prefix_id))
		", 'prefix_id');


        $prefixCategoryMap = [];
        $categoryPrefixes = $this->sourceDb->fetchAll("SELECT * FROM phc_linkdirectory_prefix_category");
        
        foreach ($categoryPrefixes AS $categoryPrefix)
        {
            $newCategoryId = $this->lookupId('ld_category', $categoryPrefix['ld_cat_id']);
            if ($newCategoryId)
            {
                $prefixCategoryMap[$categoryPrefix['prefix_id']][] = $newCategoryId;
            }
        }

        foreach ($prefixes AS $oldPrefixId => $prefix)
        {
            /** @var \xenMade\LD\Import\Data\LinkPrefix $importPrefix */
            $importPrefix = $this->newHandler('xenMade\LD:LinkPrefix');
            $importPrefix->bulkSet($this->mapKeys($prefix, [
                'display_order',
                'css_class'
            ]));

            $importPrefix->prefix_group_id = isset($mappedGroupIds[$prefix['prefix_group_id']])
                ? $mappedGroupIds[$prefix['prefix_group_id']]
                : 0;

            $importPrefix->allowed_user_group_ids = $this->mapUserGroupList($prefix['allowed_user_group_ids']);

            $importPrefix->setTitle($prefix['phrase_text']);

            if (!empty($prefixCategoryMap[$oldPrefixId]))
            {
                $importPrefix->setCategories($prefixCategoryMap[$oldPrefixId]);
            }

            if ($importPrefix->save($oldPrefixId))
            {
                $state->imported++;
            }
        }

        return $state->complete();
    }



    // ########################### STEP: Links ###############################

	public function stepLinks(StepState $state, array $stepConfig, $maxTime, $limit = 25)
	{
		$timer = new \XF\Timer($maxTime);
		
		$links = $this->getLinks($state->startAfter, $state->end, $limit);

        if (!$links)
		{
			return $state->complete();
		}

        /** @var \xenMade\LD\Import\DataHelper\Link $linkHelper */
        $linkHelper = $this->dataManager->helper('xenMade\LD:Link');

        /** @var \xenMade\LD\Import\DataHelper\Tag $tagHelper */
        $tagHelper = $this->dataManager->helper('xenMade\LD:Tag');

        $status = $this->getTable('phc_linkdirectory_field_value');

        foreach ($links AS $oldLinkId => $link)
		{
			$state->startAfter = $oldLinkId;


			/** @var \xenMade\LD\Import\Data\Link $import */
			$import = $this->newHandler('xenMade\LD:Link');
			
			$import->bulkSet($this->mapXfKeys($link, [
				'link_id' => 'ld_id',
				'title',
				'affiliate_url',
				'user_id',
				'insert_date' => 'dateline',
				'image_date' => 'logo_file_date',
				'category_id' => 'ld_cat_id',
                'click_count' => 'clicks',
                'is_vip' => 'vip',

                'rating_count',
                'rating_sum',
                'rating_avg',
                'rating_weighted',

                'prefix_id',
			]));


			$user = $this->em()->find('XF:User', $link['user_id']);


            $_reEdit = false;
            $link['pagetext'] = trim($link['pagetext']);
            if($link['pagetext'] == '')
            {
                $_reEdit = true;
            }

			$import->bulkSet([
				'username' 		=> (!empty($user->username) ? $user->username : 'Unknow'),
                'url' => $url = preg_replace('#(https?:\/\/)|(\/)+#is', '$1$2', $link['url']),
                'description' => ($link['pagetext'] ? $link['pagetext'] : '###empty###'),
                'state' => $this->decodeLinkState($link['status'])
			]);


			if ($newLinkId = $import->save($oldLinkId))
			{
			    // Image
			    $oldPath = $this->getOldImagePath($link);

                if (file_exists($oldPath) || is_readable($oldPath))
                {
                    \XF\Util\File::copyFileToAbstractedPath($oldPath, $this->getNewImagePath($newLinkId));
                }

                // FieldValues
                if($status)
                {
                    $linkFieldValues = $this->getLinkFieldValues($link['ld_id']);

                    if(!empty($linkFieldValues))
                    {
                        foreach ($linkFieldValues as &$linkFieldValue)
                        {
                            $linkFieldValue['link_id'] = $newLinkId;
                            unset($linkFieldValue['ld_id']);
                        }

                        $linkHelper->importFieldValues($linkFieldValues);
                    }
                }

                // Tags
                $tags = $this->getLinkTags($link['ld_id']);
                if($tags)
                {
                    $newTags = [];

                    foreach ($tags AS $tag)
                    {
                        $newTagId = $tagHelper->importTag($tag['tag'], 'ld_link', $newLinkId, [
                            'add_user_id' => $tag['add_user_id'],
                            'add_date' => $tag['add_date'],
                            'visible' => $tag['visible']
                        ]);
                    }
                }


                if($_reEdit)
                {
                    $link = $this->em()->find('xenMade\LD:Link', $newLinkId);

                    if($link->description == '###empty###')
                    {
                        $link->fastUpdate('description', '');
                    }
                }

				$state->imported++;
			}
			
			if ($timer->limitExceeded())
			{
				break;
			}
		}
		
		return $state->resumeIfNeeded();
	}

    public function getStepEndLinks()
    {
        return $this->sourceDb->fetchOne("SELECT MAX(ld_id) FROM phc_linkdirectory") ?: 0;
    }

	protected function getLinks($startAfter, $end, $limit)
	{
		return $this->sourceDb->fetchAllKeyed("
			SELECT link.*
			FROM phc_linkdirectory AS link
			WHERE link.ld_id > ? AND link.ld_id <= ?
			ORDER BY link.ld_id
			LIMIT {$limit}
		", 'ld_id', [$startAfter, $end]);
	}

	protected function getLinkFieldValues($linkId)
    {
        return $this->sourceDb->fetchAll("
            SELECT link_field_value.* 
            FROM phc_linkdirectory_field_value AS link_field_value
            WHERE link_field_value.ld_id = ?", $linkId);
    }

    protected function getLinkTags($linkId)
    {
        return $this->sourceDb->fetchAll("
			SELECT taglink.*, tag.*
			FROM xf_tag_content AS taglink   
			INNER JOIN xf_tag AS tag ON (tag.tag_id = taglink.tag_id)
			WHERE taglink.content_type = 'ld_links' AND taglink.content_id = ? 
		", $linkId);
    }



    // ########################### STEP: LinkRatings ###############################

    public function stepLinkRatings(StepState $state, array $stepConfig, $maxTime, $limit = 25)
    {
        $timer = new \XF\Timer($maxTime);

        $ratings = $this->getLinkRatings($state->startAfter, $state->end, $limit);

        if (!$ratings)
        {
            return $state->complete();
        }

        $this->typeMap('ld_link');

        foreach ($ratings AS $oldRatingId => $rating)
        {
            $state->startAfter = $oldRatingId;

            if(!$rating['username'])
            {
                continue;
            }

            if (!$mappedLinkId = $this->lookupId('ld_link', $rating['ld_id']))
            {
                continue;
            }

            // Prüfe ob ein Rating schon vorhanden, dann ignorieren
            $existRating = $this->db()->fetchOne('SELECT * FROM xf_xenmade_ld_link_rating WHERE user_id = ? AND link_id = ?', [$rating['user_id'], $mappedLinkId]);

            if($existRating)
            {
                continue;
            }

            /** @var \xenMade\LD\Import\Data\LinkRating $import */
            $import = $this->newHandler('xenMade\LD:LinkRating');

            $import->bulkSet($this->mapXfKeys($rating, [
                'rating_id',
                'user_id',
                'rating',
                'rating_date',
                'message',
                'rating_state',
            ]));

            $import->set('link_id', $mappedLinkId);
            $import->set('username', $rating['username']);


            if ($newRatingId = $import->save($oldRatingId))
            {
                $state->imported++;
            }

            if ($timer->limitExceeded())
            {
                break;
            }
        }

        return $state->resumeIfNeeded();
    }

    public function getStepEndLinkRatings()
    {
        return $this->sourceDb->fetchOne("SELECT MAX(rating_id) FROM phc_linkdirectory_rating") ?: 0;
    }

    protected function getLinkRatings($startAfter, $end, $limit)
    {
        return $this->sourceDb->fetchAllKeyed("
			SELECT *
			FROM phc_linkdirectory_rating
			LEFT JOIN xf_user using(user_id)
			WHERE rating_id > ? AND rating_id <= ?
			ORDER BY rating_id
			LIMIT {$limit}
		", 'rating_id', [$startAfter, $end]);
    }



    // ########################### STEP: LinkLikes ###############################

    public function stepLinksLikes(StepState $state, array $stepConfig, $maxTime)
    {
        $limit = 1000;
        $timer = new \XF\Timer($maxTime);

        $likes = $this->sourceDb->fetchAllKeyed("
			SELECT *
			FROM xf_liked_content
			WHERE like_id > ? AND like_id <= ?
				AND content_type IN ('ld_links_like')
			ORDER BY like_id
			LIMIT {$limit}
		", 'like_id', [$state->startAfter, $state->end]);

        if (!$likes)
        {
            return $state->complete();
        }

        $this->typeMap('ld_link');

        foreach ($likes AS $oldId => $like)
        {
            $state->startAfter = $oldId;

            $contentUser = $this->em()->find('XF:User', $like['content_user_id']);

            $contentUserId = $like['content_user_id'];

            if(isset($contentUser->user_id))
            {
                $contentUserId = $contentUser->user_id;
            }

            $contentId = $this->lookupId('ld_link', $like['content_id']);
            if (!$contentId)
            {
                continue;
            }

            $likedUser = $this->em()->find('XF:User', $like['like_user_id']);
            $likeUserId = $likedUser->user_id;
            if (!$likeUserId)
            {
                continue;
            }


            // ContentExists?
            $delLike = $this->db()->query('DELETE FROM xf_liked_content WHERE content_type = ? AND content_id = ? AND like_user_id = ?',
                ['ld_link', $contentId, $likeUserId]);

            if($delLike->rowsAffected())
            {
                if ($contentUserId)
                {
                    $this->db()->query("
                            UPDATE xf_user
                            SET like_count = IF(like_count > 0, like_count - 1, 0)
                            WHERE user_id = ?", $contentUserId);
                }
            }

            /** @var \xenMade\LD\Import\Data\LinkLikes $import */
            $import = $this->newHandler('xenMade\LD:LinkLikes');
            $import->preventRetainIds();

            $import->bulkSet($this->mapKeys($like, [
                'content_type',
                'like_date'
            ]));

            $import->content_id = $contentId;
            $import->content_type = 'ld_link';
            $import->like_user_id = $likeUserId;
            $import->content_user_id = $contentUserId;

            $newId = $import->save($oldId);

            if ($newId)
            {
                $state->imported++;
            }

            if ($timer->limitExceeded())
            {
                break;
            }
        }

        return $state->resumeIfNeeded();
    }

    public function getStepEndLinksLikes()
    {
        return $this->sourceDb->fetchOne("SELECT MAX(like_id) FROM xf_liked_content") ?: 0;
    }





    protected function decodeFieldDisplayGroup($status)
    {
        switch ($status)
        {
            case 'below_desc':
                return 'below_info';

            case 'above_desc':
                return 'above_info';

            default:
                return 'above_main';
        }
    }

    protected function decodeLinkState($status)
    {
        switch ($status)
        {
            case 'visible':
                return 'visible';

            case 'confirm':
            case 'blacklist':
            case 'pageunavailable':
            case 'nosoftware':
                return 'moderated';

            default:
                return 'visible';
        }
    }

    protected function decodeFieldMatchParamsGroup($value)
    {
        if (is_object(json_decode($value)))
        {
            return $value;
        }

        return [];
    }

    protected function mapUserGroupList($userGroups)
    {
        return $this->getHelper()->mapUserGroupList($userGroups);
    }

    protected function getOldImagePath($link, $format = false)
    {
        $group = floor($link['ld_id'] / 1000);

        return sprintf($this->baseConfig['data_dir'] . '/linkdirectory/%d/' . ($format ? $format . '_' : '') . $link['ld_id'] . '.jpg',
            $group
        );

    }

    protected function getNewImagePath($linkId)
    {
        return sprintf('data://ld_thumbnail/%d/%d.jpg',
            floor($linkId / 1000),
            $linkId
        );

    }

    protected function getTable($tableName)
    {
        if(empty($this->sourceDb))
            return true;

        return $this->sourceDb->fetchOne('
			SHOW TABLE STATUS WHERE Name = ?
		', $tableName);
    }
}