<?php

namespace xenMade\LD\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class Category extends AbstractController
{
	public function actionIndex(ParameterBag $params)
	{
        $category = $this->assertViewableCategory($params->category_id, $this->getCategoryViewExtraWith());

        /** @var \xenMade\LD\ControllerPlugin\Link $linkPlugin */
        $linkPlugin = $this->plugin('xenMade\LD:Link');

        $categoryParams = $linkPlugin->getCategoryListData($category);

        /** @var \XF\Tree $categoryTree */
        $categoryTree = $categoryParams['categoryTree'];
        $descendants = $categoryTree->getDescendants($category->category_id);

        $sourceCategoryIds = array_keys($descendants);
        $sourceCategoryIds[] = $category->category_id;

        $category->cacheViewableDescendents($descendants);

        $listParams = $linkPlugin->getCoreListData($sourceCategoryIds);

        $this->assertValidPage($listParams['page'], $listParams['perPage'], $listParams['total'], 'link-directory/categories', $category);

        $this->assertCanonicalUrl($this->buildLink('link-directory/categories', $category, ['page' => $listParams['page']]));

        $viewParams = [
            'category' => $category,
            'pendingApproval' => $this->filter('pending_approval', 'bool'),
        ];
        $viewParams += $categoryParams + $listParams;

        return $this->view('xenMade\LD:Category\View', 'ld_category_view', $viewParams);
	}

    public function actionAdd(ParameterBag $params)
    {
        $category = $this->assertViewableCategory($params->category_id);
        if (!$category->canAddLink($error))
        {
            return $this->noPermission($error);
        }

        // ToDo: Prüfen auf Status vor Eintrag
        if ($this->isPost())
        {
            $creator = $this->setupLinkCreate($category);

            $creator->checkForSpam();

            if (!$creator->validate($errors))
            {
                return $this->error($errors);
            }
            $this->assertNotFlooding('post');

            $creator->setBacklink();

            /** @var \xenMade\LD\Entity\Link $link */
            $link = $creator->save();
            $this->finalizeLinkCreate($creator);

            if($upgradeId = $this->filter('upgrade', 'uint'))
            {
                /** @var \xenMade\LD\Repository\LinkUpgrade $upgradeRepo */
                $upgradeRepo = $this->repository('xenMade\LD:LinkUpgrade');
                list($upgrades, $active) = $upgradeRepo->getFilteredUpgradesForList($link);

                if(empty($active) && isset($upgrades[$upgradeId]))
                {
                    return $this->redirect($this->buildLink('link-directory/upgrade', $link, ['upgrade' => $upgradeId]));
                }
            }

            if (!$link->canView())
            {
                return $this->redirect($this->buildLink('link-directory/categories', $category, [
                    'pending_approval' => 1
                ]));
            }
            else
            {
                return $this->redirect($this->buildLink('link-directory', $link));
            }
        }
        else
        {
            $link = $category->getNewLink();

            /** @var \xenMade\LD\Repository\LinkUpgrade $upgradeRepo */
            $upgradeRepo = $this->repository('xenMade\LD:LinkUpgrade');
            list($upgrades, $active) = $upgradeRepo->getFilteredUpgradesForList($link);

            $viewParams = [
                'category' => $category,
                'link' => $link,
                'prefixes' => $category->getUsablePrefixes(),
                'active' => $active,
                'upgrades' => $upgrades,
            ];

            return $this->view('xenMade\LD:Category\AddLink', 'ld_category_add_link', $viewParams);
        }
    }

    public function actionWatch(ParameterBag $params)
    {
        $category = $this->assertViewableCategory($params->category_id);

        if (!$category->canWatch($error))
        {
            return $this->noPermission($error);
        }

        $visitor = \XF::visitor();

        if ($this->isPost())
        {
            if ($this->filter('stop', 'bool'))
            {
                $action = 'delete';
                $config = [];
            }
            else
            {
                $action = 'watch';
                $config = $this->filter([
                    'notify_on' => 'str',
                    'send_alert' => 'bool',
                    'send_email' => 'bool',
                    'include_children' => 'bool'
                ]);
            }

            /** @var \xenMade\LD\Repository\CategoryWatch $watchRepo */
            $watchRepo = $this->repository('xenMade\LD:CategoryWatch');
            $watchRepo->setWatchState($category, $visitor, $action, $config);

            $redirect = $this->redirect($this->buildLink('link-directory/categories', $category));
            $redirect->setJsonParam('switchKey', $action == 'delete' ? 'watch' : 'unwatch');
            return $redirect;
        }
        else
        {
            $viewParams = [
                'category' => $category,
                'isWatched' => !empty($category->Watch[$visitor->user_id])
            ];
            return $this->view('xenMade\LD:Category\Watch', 'ld_category_watch', $viewParams);
        }
    }

    public function actionFilters(ParameterBag $params)
    {
        $category = $this->assertViewableCategory($params->category_id);

        /** @var \xenMade\LD\ControllerPlugin\Link $linkPlugin */
        $linkPlugin = $this->plugin('xenMade\LD:Link');

        return $linkPlugin->actionFilters($category);
    }



    protected function setupLinkCreate(\xenMade\LD\Entity\Category $category)
    {
        $title = $this->filter('title', 'str');
        $description = $this->plugin('XF:Editor')->fromInput('description');

        /** @var \xenMade\LD\Service\Link\Create $creator */
        $creator = $this->service('xenMade\LD:Link\Create', $category);

        $creator->setContent($title, $description);
        $creator->setUrl($this->filter('url', 'str'));


        $prefixId = $this->filter('prefix_id', 'uint');
        if ($prefixId && $category->isPrefixUsable($prefixId))
        {
            $creator->setPrefix($prefixId);
        }

        if ($category->canEditTags())
        {
            $creator->setTags($this->filter('tags', 'str'));
        }

        $creator->getLink()->set('affiliate_url', $this->filter('affiliate_url', 'str'));


        $bulkInput = $this->filter([
            'backlink_location' => 'str',
        ]);

        $creator->getLink()->bulkSet($bulkInput);


        $customFields = $this->filter('custom_fields', 'array');
        $creator->setCustomFields($customFields);

        return $creator;
    }

    protected function finalizeLinkCreate(\xenMade\LD\Service\Link\Create $creator)
    {
        $creator->sendNotifications();

        $link = $creator->getLink();

        /** @var \xenMade\LD\Thumbnail\AbstractThumbnail $ldThumbnail */
        $ldThumbnail = \XF::app()->container('ldThumbnail');
        $ldThumbnail->setLink($link);

        if(!$ldThumbnail->getThumbnail())
        {
            throw $this->errorException($ldThumbnail->getError());
        }
        else
        {
            $link->fastUpdate('image_date', \XF::$time + (86400 * 30));

            // Update Message with Image
            if($link->thread_id)
            {
                $image = $link->getImage(true);

                if($image)
                {
                    $this->app()->db()->query('UPDATE xf_post SET message = REPLACE(message, \'{image}\', "[img]'. $image . '[/img]") WHERE thread_id = ?', $link->thread_id);
                }
            }
        }

        if ($link->Category->thread_node_id && $link->Category->ThreadForum)
        {
            /** @var \xenMade\LD\Service\Link\Thread $threadCreator */
            $threadCreator = $this->service('xenMade\LD:Link\Thread', $link);

            $thread = $threadCreator->setupThreadCreation($link);
            if ($thread && $thread->validate())
            {
                $thread = $thread->save();
                $link->fastUpdate('thread_id', $thread->thread_id);
                $threadCreator->afterThreadCreated($thread);
            }
        }

        if (\XF::visitor()->user_id)
        {
            if ($link->state == 'moderated')
            {
                $this->session()->setHasContentPendingApproval();
            }
        }
    }

    protected function assertViewableCategory($categoryId, array $extraWith = [])
    {
        $visitor = \XF::visitor();

        $extraWith[] = 'Permissions|' . $visitor->permission_combination_id;

        /** @var \xenMade\LD\Entity\Category $category */
        $category = $this->em()->find('xenMade\LD:Category', $categoryId, $extraWith);
        if (!$category)
        {
            throw $this->exception($this->notFound(\XF::phrase('requested_category_not_found')));
        }

        if (!$category->canView($error))
        {
            throw $this->exception($this->noPermission($error));
        }

        return $category;
    }

    protected function getCategoryViewExtraWith()
    {
        $extraWith = [];
        $userId = \XF::visitor()->user_id;
        if ($userId)
        {
            $extraWith[] = 'Watch|' . $userId;
        }

        return $extraWith;
    }

    protected function preDispatchController($action, ParameterBag $params)
    {
        if (!\XF::visitor()->canViewLinks($error))
        {
            throw $this->exception($this->noPermission($error));
        }
    }

    /**
     * @return \xenMade\LD\Repository\Link
     */
    protected function getLinkRepo()
    {
        return $this->repository('xenMade\LD:Link');
    }

    /**
     * @return \xenMade\LD\Repository\Category
     */
    protected function getCategoryRepo()
    {
        return $this->repository('xenMade\LD:Category');
    }
}