<?php

namespace xenMade\LD\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class Link extends AbstractController
{
	public function actionIndex(ParameterBag $params)
	{
		if ($params->link_id)
		{
			return $this->rerouteController(__CLASS__, 'view', $params);
		}

        /** @var \xenMade\LD\ControllerPlugin\Link $linkPlugin */
        $linkPlugin = $this->plugin('xenMade\LD:Link');


        $categoryParams = $linkPlugin->getCategoryListData();
        $viewableCategoryIds = $categoryParams['categories']->keys();

        $listParams = $linkPlugin->getCoreListData($viewableCategoryIds);

        $this->assertValidPage($listParams['page'], $listParams['perPage'], $listParams['total'], 'link-directory');
        $this->assertCanonicalUrl($this->buildLink('link-directory', null, ['page' => $listParams['page']]));

        $viewParams = $categoryParams + $listParams;

        return $this->view('xenMade\LD:Link', 'ld_link', $viewParams);
	}

	public function actionAdd(ParameterBag $params)
    {
        /** @var \xenMade\LD\XF\Entity\User $visitor */
        $visitor = \XF::visitor();
        if (!$visitor->canAddLink($error))
        {
            return $this->noPermission($error);
        }

        $this->assertCanonicalUrl($this->buildLink('link-directory/add'));

        $categoryRepo = $this->getCategoryRepo();

        $categories = $categoryRepo->getViewableCategories();
        $canAdd = false;

        foreach ($categories AS $category)
        {
            /** @var \xenMade\LD\Entity\Category $category */
            if ($category->canAddLink())
            {
                $canAdd = true;
                break;
            }
        }

        if (!$canAdd)
        {
            return $this->noPermission();
        }

        $categoryTree = $categoryRepo->createCategoryTree($categories);
        $categoryTree = $categoryTree->filter(null, function($id, \xenMade\LD\Entity\Category $category, $depth, $children)
        {
            if ($children)
            {
                return true;
            }

            if ($category->canAddLink())
            {
                return true;
            }

            return false;
        });

        $categoryExtras = $categoryRepo->getCategoryListExtras($categoryTree);

        $viewParams = [
            'categoryTree' => $categoryTree,
            'categoryExtras' => $categoryExtras
        ];

        return $this->view('xenMade\LD:Link\Chooser', 'ld_link_category_chooser', $viewParams);
    }

    public function actionFilters()
    {
        /** @var \xenMade\LD\ControllerPlugin\Link $linkPlugin */
        $linkPlugin = $this->plugin('xenMade\LD:Link');

        return $linkPlugin->actionFilters();
    }

    public function actionView(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);

        $hasRated = false;
        if(!empty($link->Rating[\XF::visitor()->user_id]))
        {
            $hasRated = true;
        }

        $this->assertCanonicalUrl($this->buildLink('link-directory', $link));

        $commentRepo = $this->getLinkCommentRepo();
        $commendFinder = $commentRepo->findLatestLinkCommentOnLink($link);
        $comment = $commendFinder->fetchOne();


        $rating = [];
        if($link->canViewRatings())
        {
            $ratingRepo = $this->getLinkRatingRepo();
            $ratingFinder = $ratingRepo->findLatestRatingOnLink($link);
            $rating = $ratingFinder->fetchOne();
        }

        $viewParams = [
            'link' => $link,
            'category' => $link->Category,
            'has_rated' => $hasRated,

            'comment' => $comment,
            'rating' => $rating,
        ];

        return $this->view('xenMade\LD:Link\View', 'ld_link_view', $viewParams);
    }

    public function actionTags(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);

        if (!$link->canEditTags($error))
        {
            return $this->noPermission($error);
        }

        /** @var \XF\Service\Tag\Changer $tagger */
        $tagger = $this->service('XF:Tag\Changer', 'ld_link', $link);

        if ($this->isPost())
        {
            $tagger->setEditableTags($this->filter('tags', 'str'));
            if ($tagger->hasErrors())
            {
                return $this->error($tagger->getErrors());
            }

            $tagger->save();

            return $this->redirect($this->buildLink('link-directory', $link));
        }
        else
        {
            $grouped = $tagger->getExistingTagsByEditability();

            $viewParams = [
                'link' => $link,
                'category' => $link->Category,
                'editableTags' => $grouped['editable'],
                'uneditableTags' => $grouped['uneditable']
            ];
            return $this->view('xenMade\LD:Link\Tags', 'ld_link_tags', $viewParams);
        }
    }

    public function actionPrefixes(ParameterBag $params)
    {
        $this->assertPostOnly();

        $categoryId = $this->filter('val', 'uint');

        /** @var \xenMade\LD\Entity\Category $category */
        $category = $this->em()->find('xenMade\LD:Category', $categoryId, 'Permissions|' . \XF::visitor()->permission_combination_id);
        if (!$category)
        {
            return $this->notFound(\XF::phrase('requested_category_not_found'));
        }

        if (!$category->canView($error))
        {
            return $this->noPermission($error);
        }

        $viewParams = [
            'category' => $category,
            'prefixes' => $category->getUsablePrefixes()
        ];
        return $this->view('xenMade\LD:Category\Prefixes', 'ld_category_prefixes', $viewParams);
    }

    public function actionEdit(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canEdit($error))
        {
            return $this->noPermission($error);
        }

        $category = $link->Category;

        /** @var \xenMade\LD\Repository\LinkUpgrade $upgradeRepo */
        $upgradeRepo = $this->repository('xenMade\LD:LinkUpgrade');
        list($upgrades, $active) = $upgradeRepo->getFilteredUpgradesForList($link);

        if ($this->isPost())
        {
            $editor = $this->setupLinkCreate($link);
            $editor->checkForSpam();

            if (!$editor->validate($errors))
            {
                return $this->error($errors);
            }

            $editor->setBacklink();

            $link = $editor->save();

            /** @var \xenMade\LD\Thumbnail\AbstractThumbnail $ldThumbnail */
            $ldThumbnail = \XF::app()->container('ldThumbnail');
            $ldThumbnail->setLink($link);

            if(!$ldThumbnail->getThumbnail())
            {
                throw $this->errorException($ldThumbnail->getError());
            }

            $link->fastUpdate([
                    'image_date' => \XF::$time + (86400 * 30),
                    'last_update' => \XF::$time
                ]
            );

            if($upgradeId = $this->filter('upgrade', 'uint'))
            {
                /** @var \xenMade\LD\Repository\LinkUpgrade $upgradeRepo */
                $upgradeRepo = $this->repository('xenMade\LD:LinkUpgrade');
                list($upgrades, $active) = $upgradeRepo->getFilteredUpgradesForList($link);

                if(empty($active) && isset($upgrades[$upgradeId]))
                {
                    return $this->redirect($this->buildLink('link-directory/upgrade', $link, ['upgrade' => $upgradeId]));
                }
            }

            return $this->redirect($this->buildLink('link-directory', $link));
        }
        else
        {
            $viewParams = [
                'link' => $link,
                'category' => $category,
                'prefixes' => $category->getUsablePrefixes($link->prefix_id),
                'active' => $active,
                'upgrades' => $upgrades,
            ];

            return $this->view('xenMade\LD:Link\Edit', 'ld_link_edit', $viewParams);
        }
    }

    public function actionReport(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canReport($error))
        {
            return $this->noPermission($error);
        }

        /** @var \XF\ControllerPlugin\Report $reportPlugin */
        $reportPlugin = $this->plugin('XF:Report');
        return $reportPlugin->actionReport('ld_link', $link,
            $this->buildLink('link-directory/report', $link),
            $this->buildLink('link-directory', $link)
        );
    }

    public function actionUndelete(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canUndelete($error))
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            if ($link->state == 'deleted')
            {
                $link->state = 'visible';
                $link->save();
            }

            return $this->redirect($this->buildLink('link-directory', $link));
        }
        else
        {
            $viewParams = [
                'link' => $link,
                'category' => $link->Category
            ];
            return $this->view('xenMade\LD:Link\Undelete', 'ld_link_undelete', $viewParams);
        }
    }

    public function actionApprove(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canApproveUnapprove($error))
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            /** @var \xenMade\LD\Service\Link\Approve $approver */
            $approver = \XF::service('xenMade\LD:Link\Approve', $link);
            $approver->setNotifyRunTime(1);
            $approver->approve();

            return $this->redirect($this->buildLink('link-directory', $link));
        }
        else
        {
            $viewParams = [
                'link' => $link,
                'category' => $link->Category
            ];
            return $this->view('xenMade\LD:Link\Approve', 'ld_link_approve', $viewParams);
        }
    }

    public function actionDelete(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canDelete('soft', $error))
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            $type = $this->filter('hard_delete', 'bool') ? 'hard' : 'soft';
            $reason = $this->filter('reason', 'str');

            if (!$link->canDelete($type, $error))
            {
                return $this->noPermission($error);
            }

            /** @var \xenMade\LD\Service\Link\Delete $deleter */
            $deleter = $this->service('xenMade\LD:Link\Delete', $link);

            if ($this->filter('author_alert', 'bool'))
            {
                $deleter->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
            }

            $deleter->delete($type, $reason);

            $this->plugin('XF:InlineMod')->clearIdFromCookie('ld_link', $link->link_id);

            return $this->redirect($this->buildLink('link-directory/categories', $link->Category));
        }
        else
        {
            $viewParams = [
                'link' => $link,
                'category' => $link->Category
            ];
            return $this->view('XF:Link\Delete', 'ld_link_delete', $viewParams);
        }
    }

    public function actionReassign(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canReassign($error))
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            $user = $this->em()->findOne('XF:User', ['username' => $this->filter('username', 'str')]);
            if (!$user)
            {
                return $this->error(\XF::phrase('requested_user_not_found'));
            }

            $canTargetView = \XF::asVisitor($user, function() use ($link)
            {
                return $link->canView();
            });
            if (!$canTargetView)
            {
                return $this->error(\XF::phrase('ld_new_owner_must_be_able_to_view_this_link'));
            }

            /** @var \xenMade\LD\Service\Link\Reassign $reassigner */
            $reassigner = $this->service('xenMade\LD:Link\Reassign', $link);

            if ($this->filter('alert', 'bool'))
            {
                $reassigner->setSendAlert(true, $this->filter('alert_reason', 'str'));
            }

            $reassigner->reassignTo($user);

            return $this->redirect($this->buildLink('link-directory', $link));
        }
        else
        {
            $viewParams = [
                'link' => $link,
                'category' => $link->Category
            ];
            return $this->view('xenMade\LD:Link\Reassign', 'ld_link_reassign', $viewParams);
        }
    }

    public function actionMove(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canMove($error))
        {
            return $this->noPermission($error);
        }

        $category = $link->Category;

        if ($this->isPost())
        {
            $targetCategoryId = $this->filter('target_category_id', 'uint');

            /** @var \xenMade\LD\Entity\Category $targetCategory */
            $targetCategory = $this->app()->em()->find('xenMade\LD:Category', $targetCategoryId);
            if (!$targetCategory || !$targetCategory->canView())
            {
                return $this->error(\XF::phrase('requested_category_not_found'));
            }

            $this->linkMove($link)->move($targetCategory);

            return $this->redirect($this->buildLink('link-directory', $link));
        }
        else
        {
            $categoryRepo = $this->getCategoryRepo();
            $categories = $categoryRepo->getViewableCategories();

            $viewParams = [
                'link' => $link,
                'category' => $category,
                'prefixes' => $category->getUsablePrefixes(),
                'categoryTree' => $categoryRepo->createCategoryTree($categories)
            ];
            return $this->view('xenMade\LD:Link\Move', 'ld_link_move', $viewParams);
        }
    }

    public function actionWatch(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canWatch($error))
        {
            return $this->noPermission($error);
        }

        $visitor = \XF::visitor();

        if ($this->isPost())
        {
            if ($this->filter('stop', 'bool'))
            {
                $action = 'delete';
                $config = [];
            }
            else
            {
                $action = 'watch';
                $config = [
                    'email_subscribe' => $this->filter('email_subscribe', 'bool')
                ];
            }

            /** @var \xenMade\LD\Repository\LinkWatch $watchRepo */
            $watchRepo = $this->repository('xenMade\LD:LinkWatch');
            $watchRepo->setWatchState($link, $visitor, $action, $config);

            $redirect = $this->redirect($this->buildLink('link-directory', $link));
            $redirect->setJsonParam('switchKey', $action == 'delete' ? 'watch' : 'unwatch');
            return $redirect;
        }
        else
        {
            $viewParams = [
                'link' => $link,
                'isWatched' => !empty($link->Watch[$visitor->user_id]),
                'category' => $link->Category
            ];
            return $this->view('xenMade\LD:Link\Watch', 'ld_link_watch', $viewParams);
        }
    }

    public function actionIp(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        $breadcrumbs = $link->getBreadcrumbs();

        /** @var \XF\ControllerPlugin\Ip $ipPlugin */
        $ipPlugin = $this->plugin('XF:Ip');
        return $ipPlugin->actionIp($link, $breadcrumbs);
    }

    public function actionClaim(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);

        if(!$link->canClaim())
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            if(!$this->filter('message', 'str'))
            {
                throw $this->exception($this->error(\XF::phrase('ld_please_provide_information_in_order_to_claim_link_ownership')));
            }

            /** @var \XF\ControllerPlugin\Report $reportPlugin */
            $reportPlugin = $this->plugin('XF:Report');
            return $reportPlugin->actionReport('ld_link_claim', $link,
                $this->buildLink('link-directory/report', $link),
                $this->buildLink('link-directory', $link)
            );
        }
        else
        {
            $viewParams = [
                'link' => $link,
            ];

            return $this->view('xenMade\LD:Link\Claim', 'ld_link_claim', $viewParams);
        }
    }

    public function actionWarn(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);

        if (!$link->canWarn($error))
        {
            return $this->noPermission($error);
        }

        $breadcrumbs = $link->Category->getBreadcrumbs();

        /** @var \XF\ControllerPlugin\Warn $warnPlugin */
        $warnPlugin = $this->plugin('XF:Warn');
        return $warnPlugin->actionWarn(
            'ld_link', $link,
            $this->buildLink('link-directory/warn', $link),
            $breadcrumbs
        );
    }

    public function actionAddComment(ParameterBag $params)
    {
        $this->assertPostOnly();
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canAddComment())
        {
            return $this->noPermission();
        }

        $creator = $this->setupLinkCommentCreate($link);
        $creator->checkForSpam();

        if (!$creator->validate($errors))
        {
            return $this->error($errors);
        }
        $this->assertNotFlooding('post');
        $linkComment = $creator->save();

        $this->finalizeLinkCommentCreate($creator);

        if ($this->filter('_xfWithData', 'bool') && $this->request->exists('last_date') && $linkComment->canView())
        {
            $linkCommentRepo = $this->getLinkCommentRepo();

            $limit = 3;
            $lastDate = $this->filter('last_date', 'uint');
            $style = $this->filter('style', 'str');
            $context = $this->filter('context', 'str');
            $firstUnshownLinkComment = null;

            if ($context == 'all')
            {
                /** @var \XF\Mvc\Entity\Finder $linkCommentList */
                $linkCommentList = $linkCommentRepo->findNewestLinkComments($lastDate)->forFullView();
                $linkComments = $linkCommentList->fetch($limit)->filterViewable();
            }
            else
            {
                /** @var \XF\Mvc\Entity\Finder $linkCommentList */
                $linkCommentList = $linkCommentRepo->findNewestLinkCommentsOnLink($link, $lastDate)->forFullView();
                $linkComments = $linkCommentList->fetch($limit + 1)->filterViewable();

                if ($linkComments->count() > $limit)
                {
                    $firstUnshownLinkComment = $linkComments->last();
                    $linkComments = $linkComments->pop();
                }
            }

            $linkComments = $linkComments->reverse(true);

            $viewParams = [
                'link' => $link,
                'style' => $style,
                'linkComments' => $linkComments,
                'firstUnshownLinkComment' => $firstUnshownLinkComment
            ];

            $view = $this->view('xenMade\LD:LinkComment\NewLinkComment', 'ld_link_comment_new_comment', $viewParams);
            $view->setJsonParam('lastDate', $linkComments->last()->post_date);
            return $view;
        }
        else
        {
            return $this->redirect($this->buildLink('link-directory', $link), \XF::phrase('ld_your_link_comment_has_been_posted'));
        }
    }

    public function actionRate(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);

        if (!$link->canRate($error))
        {
            return $this->noPermission($error);
        }

        if ($link->Rating[\XF::visitor()->user_id])
        {
            return $this->noPermission(\XF::phrase('ld_you_have_already_rated_this_link'));
        }

        if ($this->isPost())
        {
            $rater = $this->setupLinkRate($link);
            $rater->checkForSpam();

            if (!$rater->validate($errors))
            {
                return $this->error($errors);
            }

            $rater->save();

            return $this->redirect($this->buildLink('link-directory/ratings', $link));
        }
        else
        {
            $viewParams = [
                'link' => $link,
                'category' => $link->Category
            ];

            return $this->view('xenMade\LD:Link\Rate', 'ld_link_rate', $viewParams);
        }
    }

    public function actionVip(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canVIP($error))
        {
            return $this->noPermission($error);
        }

        if ($link->is_vip)
        {
            $link->is_vip = false;
        }
        else
        {
            $link->is_vip = true;
        }

        $link->save();

        return $this->redirect($this->buildLink('link-directory', $link));
    }

    public function actionUpgrade(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);


        $purchasable = $this->em()->find('XF:Purchasable', 'ld_link_upgrade', 'AddOn');
        if (!$purchasable->isActive())
        {
            return $this->message(\XF::phrase('ld_no_link_upgrades_can_be_purchased_at_this_time'));
        }

        /** @var \xenMade\LD\Repository\LinkUpgrade $upgradeRepo */
        $upgradeRepo = $this->repository('xenMade\LD:LinkUpgrade');
        list($upgrades, $active) = $upgradeRepo->getFilteredUpgradesForList($link);

        if (!$upgrades && !$active)
        {
            return $this->message(\XF::phrase('ld_no_link_upgrades_can_be_purchased_at_this_time'));
        }

        if ($link->state != 'visible')
        {
            return $this->error(\XF::phrase('ld_link_upgrades_cannot_be_purchased_link_invisible'));
        }

        $upgradeId = $this->filter('upgrade', 'uint');
        if(!isset($upgrades[$upgradeId]))
        {
            return $this->message(\XF::phrase('ld_no_link_upgrades_can_be_purchased_at_this_time'));
        }

        $category = $link->Category;

        $viewParams = [
            'link' => $link,
            'category' => $category,
            'prefixes' => $category->getUsablePrefixes($link->prefix_id),
            'upgrade' => $upgrades[$upgradeId],
        ];

        return $this->view('xenMade\LD:Link\Edit', 'ld_link_upgrade', $viewParams);
    }

    public function actionUpgradePurchase()
    {
        return $this->view('xenMade\LD:Link\UpgradePurchase', 'ld_link_upgrade_purchase', []);
    }

    public function actionLike(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);
        if (!$link->canLike($error))
        {
            return $this->noPermission($error);
        }

        /** @var \XF\ControllerPlugin\Like $likePlugin */
        $likePlugin = $this->plugin('XF:Like');
        return $likePlugin->actionToggleLike(
            $link,
            $this->buildLink('link-directory/like', $link),
            $this->buildLink('link-directory', $link),
            $this->buildLink('link-directory/likes', $link)
        );
    }

    public function actionLikes(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);

        /** @var \XF\ControllerPlugin\Like $likePlugin */
        $likePlugin = $this->plugin('XF:Like');
        return $likePlugin->actionLikes(
            $link,
            ['link-directory/likes', $link]
        );
    }

    public function actionCreateThread(ParameterBag $params)
    {
        $link = $this->assertViewableLink($params->link_id);

        if(!$link->Category->thread_node_id || !$link->Category->ThreadForum && $link->hasThread())
        {
            throw $this->exception($this->noPermission());
        }

        if (!$link->canEdit())
        {
            return $this->noPermission($error);
        }

        if ($this->isPost())
        {
            /** @var \xenMade\LD\Service\Link\Thread $threadCreator */
            $threadCreator = $this->service('xenMade\LD:Link\Thread', $link);

            $thread = $threadCreator->setupThreadCreation($link);
            if ($thread && $thread->validate())
            {
                $thread = $thread->save();
                $link->fastUpdate('thread_id', $thread->thread_id);
                $threadCreator->afterThreadCreated($thread);
            }

            return $this->redirect($this->buildLink('link-directory', $link));
        }
        else
        {
            $viewParams = [
                'link' => $link
            ];

            return $this->view('XF:Link\CreateThread', 'ld_link_create_thread', $viewParams);
        }
    }




    public function actionRatings(ParameterBag $params)
    {
        if (!$params->link_id)
        {
            return $this->redirectPermanently($this->buildLink('link-directory/latest-ratings'));
        }

        $link = $this->assertViewableLink($params->link_id);

        if (!$link->canViewRatings($error))
        {
            return $this->noPermission($error);
        }


        $page = $this->filterPage();
        $perPage = $this->options()->messagesPerPage;

        /** @var \xenMade\LD\Repository\LinkRating $ratingRepo */
        $ratingRepo = $this->repository('xenMade\LD:LinkRating');
        $ratingFinder = $ratingRepo->findRatingsInLink($link);


        $total = $ratingFinder->total();

        $this->assertValidPage($page, $perPage, $total, 'link-directory/ratings', $link);
        $this->assertCanonicalUrl($this->buildLink('link-directory/ratings', $link, ['page' => $page]));

        $ratingFinder->forFullView()->limitByPage($page, $perPage);
        $ratings = $ratingFinder->fetch();

        $hasRated = false;
        if(!empty($link->Rating[\XF::visitor()->user_id]))
        {
            $hasRated = true;
        }

        $viewParams = [
            'link' => $link,
            'ratings' => $ratings,

            'page' => $page,
            'perPage' => $perPage,
            'total' => $total,
            'has_rated' => $hasRated,
        ];

        return $this->view('xenMade\LD:Link\Ratings', 'ld_link_ratings', $viewParams);
    }

    public function actionComments(ParameterBag $params)
    {
        if (!$params->link_id)
        {
            return $this->redirectPermanently($this->buildLink('link-directory/latest-ratings'));
        }

        if (!\XF::visitor()->canViewLinkComments($error))
        {
            return $this->noPermission($error);
        }

        $link = $this->assertViewableLink($params->link_id);


        $page = $this->filterPage();
        $perPage = $this->options()->messagesPerPage;


        $commentRepo = $this->getLinkCommentRepo();

        $commendFinder = $commentRepo->findCommentsOnLink($link, [
            'allowOwnPending' => $this->hasContentPendingApproval()
        ]);

        $total = $commendFinder->total();

        $this->assertValidPage($page, $perPage, $total, 'link-directory/comments', $link);
        $this->assertCanonicalUrl($this->buildLink('link-directory/comments', $link, ['page' => $page]));


        $commendFinder->forFullView()->limitByPage($page, $perPage);
        $comments = $commendFinder->fetch();

        $hasRated = false;
        if(!empty($link->Rating[\XF::visitor()->user_id]))
        {
            $hasRated = true;
        }

        $viewParams = [
            'link' => $link,
            'comments' => $comments,

            'page' => $page,
            'perPage' => $perPage,
            'total' => $total,
            'has_rated' => $hasRated,
        ];

        return $this->view('xenMade\LD:Link\Ratings', 'ld_link_comments', $viewParams);
    }


    public function actionPagedata()
    {
  		$this->assertPostOnly();

        $url = $this->filter('url', 'str');

        /** @var \xenMade\LD\ControllerPlugin\PageData $pageDataPlugin */
        $pageDataPlugin = $this->plugin('xenMade\LD:PageData');

        $data = $pageDataPlugin->fetchData($url);

  		$view = $this->view('xenMade\LD:Link\PageData', '', []);
		$view->setJsonParams($data);

		return $view;
    }

    public function actionClicker()
    {
        $ldId = $this->filter('ldid', 'uint');
        $link = $this->assertViewableLink($ldId);

        $link->fastUpdate('click_count', $link->click_count + 1);

        $view = $this->view('xenMade\LD:Link\Clicker', '', []);
        $view->setJsonParams([]);
        return $view;
    }



    protected function finalizeLinkCommentCreate(\xenMade\LD\Service\LinkComment\Creator $creator)
    {
        $creator->sendNotifications();

        $linkComment = $creator->getLinkComment();

        if (\XF::visitor()->user_id)
        {
            if ($linkComment->message_state == 'moderated')
            {
                $this->session()->setHasContentPendingApproval();
            }
        }
    }

    protected function assertViewableLink($linkId, array $extraWith = [])
    {
        $visitor = \XF::visitor();

        $extraWith[] = 'User';
        $extraWith[] = 'Category';
        $extraWith[] = 'Category.Permissions|' . $visitor->permission_combination_id;

        if ($visitor->user_id)
        {
            $extraWith[] = 'Watch|' . $visitor->user_id;
            $extraWith[] = 'Rating|' . $visitor->user_id;
        }

        /** @var \xenMade\LD\Entity\Link $link */
        $link = $this->em()->find('xenMade\LD:Link', $linkId, $extraWith);

        if (!$link)
        {
            throw $this->exception($this->notFound(\XF::phrase('ld_requested_link_not_found')));
        }

        if (!$link->canView($error))
        {
            throw $this->exception($this->noPermission($error));
        }

        return $link;
    }

    protected function assertViewableCategory($categoryId, array $extraWith = [])
    {
        $visitor = \XF::visitor();

        $extraWith[] = 'Permissions|' . $visitor->permission_combination_id;

        /** @var \xenMade\LD\Entity\Category $category */
        $category = $this->em()->find('xenMade\LD:Category', $categoryId, $extraWith);
        if (!$category)
        {
            throw $this->exception($this->notFound(\XF::phrase('requested_category_not_found')));
        }

        if (!$category->canView($error))
        {
            throw $this->exception($this->noPermission($error));
        }

        return $category;
    }

    protected function linkMove(\xenMade\LD\Entity\Link $link)
    {
        $options = $this->filter([
            'notify_watchers' => 'bool',
            'author_alert' => 'bool',
            'author_alert_reason' => 'str',
            'prefix_id' => 'uint'
        ]);

        /** @var \xenMade\LD\Service\Link\Move $mover */
        $mover = $this->service('xenMade\LD:Link\Move', $link);

        if ($options['author_alert'])
        {
            $mover->setSendAlert(true, $options['author_alert_reason']);
        }

        if ($options['notify_watchers'])
        {
            $mover->setNotifyWatchers();
        }

        if ($options['prefix_id'] !== null)
        {
            $mover->setPrefix($options['prefix_id']);
        }

        $mover->addExtraSetup(function($link)
        {
            $link->title = $this->filter('title', 'str');
        });

        return $mover;
    }

    protected function preDispatchController($action, ParameterBag $params)
    {
        if (!\XF::visitor()->canViewLinks($error))
        {
            throw $this->exception($this->noPermission($error));
        }
    }

    protected function setupLinkCreate(\xenMade\LD\Entity\Link $link)
    {
        /** @var \xenMade\LD\Service\Link\Edit $editer */
        $editer = $this->service('xenMade\LD:Link\Edit', $link);

        $prefixId = $this->filter('prefix_id', 'uint');
        if ($prefixId != $link->prefix_id && !$link->Category->isPrefixUsable($prefixId))
        {
            $prefixId = 0;
        }

        $description = $this->plugin('XF:Editor')->fromInput('description');

        $editer->setPrefix($prefixId);

        $editer->setTitle($this->filter('title', 'str'));
        $editer->setDescription($description);
        $editer->setUrl($this->filter('url', 'str'));

        $editer->getLink()->set('affiliate_url', $this->filter('affiliate_url', 'str'));

        $bulkInput = $this->filter([
            'backlink_location' => 'str',
        ]);

        $editer->getLink()->bulkSet($bulkInput);

        $customFields = $this->filter('custom_fields', 'array');
        $editer->setCustomFields($customFields);

        if ($this->filter('author_alert', 'bool') && $link->canSendModeratorActionAlert())
        {
            $editer->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
        }

        return $editer;
    }

    protected function setupLinkCommentCreate(\xenMade\LD\Entity\Link $link)
    {
        $message = $this->filter('message', 'str');

        /** @var \xenMade\LD\Service\LinkComment\Creator $creator */
        $creator = $this->service('xenMade\LD:LinkComment\Creator', $link);
        $creator->setContent($message);

        return $creator;
    }

    protected function setupLinkRate(\xenMade\LD\Entity\Link $link)
    {
        /** @var \xenMade\LD\Service\Link\Rating $rater */
        $rater = $this->service('xenMade\LD:Link\Rating', $link);

        $input = $this->filter([
            'rating' => 'uint',
            'message' => 'str'
        ]);

        $rater->setRating($input['rating'], $input['message']);

        return $rater;
    }


    /**
     * @return \xenMade\LD\Repository\Link
     */
    protected function getLinkRepo()
    {
        return $this->repository('xenMade\LD:Link');
    }

    /**
     * @return \xenMade\LD\Repository\Category
     */
    protected function getCategoryRepo()
    {
        return $this->repository('xenMade\LD:Category');
    }


    /**
     * @return \xenMade\LD\Repository\LinkComment
     */
    protected function getLinkCommentRepo()
    {
        return $this->repository('xenMade\LD:LinkComment');
    }


    /**
     * @return \xenMade\LD\Repository\LinkRating
     */
    protected function getLinkRatingRepo()
    {
        return $this->repository('xenMade\LD:LinkRating');
    }
}