<?php

namespace xenMade\LD\Purchasable;

use XF\Payment\CallbackState;
use XF\Purchasable\AbstractPurchasable;
use XF\Purchasable\Purchase;

class LinkUpgrade extends AbstractPurchasable
{
    protected $link = null;

    public function getTitle()
    {
        return \XF::phrase('ld_link_upgrades');
    }

    public function getPurchaseFromRequest(\XF\Http\Request $request, \XF\Entity\User $purchaser, &$error = null)
    {
        $profileId = $request->filter('payment_profile_id', 'uint');


        $paymentProfile = \XF::em()->find('XF:PaymentProfile', $profileId);
        if (!$paymentProfile || !$paymentProfile->active)
        {
            $error = \XF::phrase('please_choose_valid_payment_profile_to_continue_with_your_purchase');
            return false;
        }

        $linkId = $request->filter('link_id', 'uint');
        $link = \XF::em()->find('xenMade\LD:Link', $linkId);

        if (!$link || $link->state != 'visible')
        {
            $error = \XF::phrase('this_item_cannot_be_purchased_at_moment');
            return false;
        }

        $upgradeId = $request->filter('upgrade_id', 'uint');
        $upgrade = \XF::em()->find('xenMade\LD:LinkUpgrade', $upgradeId);

        if (!$upgrade || !$upgrade->canPurchase($link))
        {
            $error = \XF::phrase('this_item_cannot_be_purchased_at_moment');
            return false;
        }

        if (!in_array($profileId, $upgrade->payment_profile_ids))
        {
            $error = \XF::phrase('selected_payment_profile_is_not_valid_for_this_purchase');
            return false;
        }


        $this->link = $link;
        return $this->getPurchaseObject($paymentProfile, $upgrade, $purchaser);
    }

    public function getPurchasableFromExtraData(array $extraData)
    {
        $output = [
            'link' => '',
            'title' => '',
            'purchasable' => null
        ];

        $upgrade = \XF::em()->find('xenMade\LD:LinkUpgrade', $extraData['upgrade_id']);
        if ($upgrade)
        {
            $output['link'] = \XF::app()->router('admin')->buildLink('link-directory/upgrades/edit', $upgrade);
            $output['title'] = $upgrade->title;
            $output['purchasable'] = $upgrade;
        }
        return $output;
    }

    public function getPurchaseFromExtraData(array $extraData, \XF\Entity\PaymentProfile $paymentProfile, \XF\Entity\User $purchaser, &$error = null)
    {
        $upgrade = $this->getPurchasableFromExtraData($extraData);
        if (!$upgrade['purchasable'] || !$upgrade['purchasable']->canPurchase())
        {
            $error = \XF::phrase('this_item_cannot_be_purchased_at_moment');
            return false;
        }

        if (!in_array($paymentProfile->payment_profile_id, $upgrade['purchasable']->payment_profile_ids))
        {
            $error = \XF::phrase('selected_payment_profile_is_not_valid_for_this_purchase');
            return false;
        }

        return $this->getPurchaseObject($paymentProfile, $upgrade['purchasable'], $purchaser);
    }

    public function getPurchaseObject(\XF\Entity\PaymentProfile $paymentProfile, $purchasable, \XF\Entity\User $purchaser)
    {
        $purchase = new Purchase();

        $purchase->title = \XF::phrase('ld_link_upgrade') . ': ' . $purchasable->title . ' (' . $purchaser->username . ')';
        $purchase->description = $purchasable->description;
        $purchase->cost = $purchasable->cost_amount;
        $purchase->currency = $purchasable->cost_currency;
        $purchase->recurring = ($purchasable->recurring && $purchasable->length_unit);
        $purchase->lengthAmount = $purchasable->length_amount;
        $purchase->lengthUnit = $purchasable->length_unit;
        $purchase->purchaser = $purchaser;
        $purchase->paymentProfile = $paymentProfile;
        $purchase->purchasableTypeId = $this->purchasableTypeId;
        $purchase->purchasableId = $purchasable->upgrade_id;
        $purchase->purchasableTitle = $purchasable->title;
        $purchase->extraData = [
            'upgrade_id' => $purchasable->upgrade_id,
            'link_id' => $this->link->link_id
        ];

        $router = \XF::app()->router('public');

        $purchase->returnUrl = $router->buildLink('canonical:link-directory/upgrade-purchase', $this->link);
        $purchase->cancelUrl = $router->buildLink('canonical:link-directory/edit', $this->link);

        return $purchase;
    }



    public function completePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest = null;
            $upgradeId = $state->upgrade->upgrade_id;
            $upgradeRecordId = $state->upgradeRecordId;
        }
        else
        {
            $purchaseRequest = $state->getPurchaseRequest();
            $upgradeId = $purchaseRequest->extra_data['upgrade_id'];
            $upgradeRecordId = isset($purchaseRequest->extra_data['upgrade_record_id'])
                ? $purchaseRequest->extra_data['upgrade_record_id']
                : null;
        }

        $paymentResult = $state->paymentResult;

        $link = \XF::em()->find('xenMade\LD:Link', $purchaseRequest->extra_data['link_id']);

        /** @var \xenMade\LD\Repository\LinkUpgrade $upgradeRepo */
        $upgradeRepo = \XF::repository('xenMade\LD:LinkUpgrade');
        list($upgrades, $active) = $upgradeRepo->getFilteredUpgradesForList($link);

        $upgrade = $upgrades[$upgradeId];

        /** @var \xenMade\LD\Service\Link\Upgrade $upgradeService */
        $upgradeService = \XF::app()->service('xenMade\LD:Link\Upgrade', $upgrade, $link);

        if ($state->extraData && is_array($state->extraData))
        {
            $upgradeService->setExtraData($state->extraData);
        }

        $activeUpgrade = null;

        switch ($paymentResult)
        {
            case CallbackState::PAYMENT_RECEIVED:
                $upgradeService->setPurchaseRequestKey($state->requestKey);
                $activeUpgrade = $upgradeService->upgrade();

                $state->logType = 'payment';
                $state->logMessage = 'Payment received, upgraded/extended.';
                break;

            case CallbackState::PAYMENT_REINSTATED:
                if ($upgradeRecordId)
                {
                    $existingRecord = \XF::em()->find('xenMade\LD:LinkUpgradeActive', $upgradeRecordId);
                    $endColumn = 'end_date';

                    if (!$existingRecord)
                    {
                        $existingRecord = \XF::em()->find('xenMade\LD:LinkUpgradeExpired', $upgradeRecordId);
                        $endColumn = 'original_end_date';
                    }
                    if ($existingRecord)
                    {
                        $upgradeService->setEndDate($existingRecord->$endColumn);
                    }
                    $upgradeService->ignoreUnpurchasable(true);
                    $activeUpgrade = $upgradeService->upgrade();

                    $state->logType = 'payment';
                    $state->logMessage = 'Reversal cancelled, upgrade reactivated.';
                }
                else
                {
                    // We can't reinstate the upgrade because there doesn't appear to be an existing record.
                    $state->logType = 'info';
                    $state->logMessage = 'OK, no action.';
                }
                break;
        }

        if ($activeUpgrade && $purchaseRequest)
        {
            $extraData = $purchaseRequest->extra_data;
            $extraData['upgrade_record_id'] = $activeUpgrade->upgrade_record_id;
            $purchaseRequest->extra_data = $extraData;
            $purchaseRequest->save();
        }
    }

    public function reversePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest = null;
            $upgradeId = $state->upgrade->upgrade_id;
        }
        else
        {
            $purchaseRequest = $state->getPurchaseRequest();
            $upgradeId = $purchaseRequest->extra_data['upgrade_id'];
        }

        $purchaser = $state->getPurchaser();

        $upgrade = \XF::em()->find('xenMade\LD:LinkUpgrade', $upgradeId, 'Active|' . $purchaser->user_id);

        /** @var \xenMade\LD\Service\Link\LinkDowngrade $downgradeService */
        $downgradeService = \XF::app()->service('xenMade\LD:Link\LinkDowngrade', $upgrade, $purchaser);
        $downgradeService->setSendAlert(false);
        $downgradeService->downgrade();

        $state->logType = 'cancel';
        $state->logMessage = 'Payment refunded/reversed, downgraded.';
    }

    public function getPurchasablesByProfileId($profileId)
    {
        $finder = \XF::finder('xenMade\LD:LinkUpgrade');

        $quotedProfileId = $finder->quote($profileId);
        $columnName = $finder->columnSqlName('payment_profile_ids');

        $router = \XF::app()->router('admin');
        $upgrades = $finder->whereSql("FIND_IN_SET($quotedProfileId, $columnName)")->fetch();
        return $upgrades->pluck(function(\xenMade\LD\Entity\LinkUpgrade $upgrade, $key) use ($router)
        {
            return ['upgrade_' . $key, [
                'title' => $this->getTitle() . ': ' . $upgrade->title,
                'link' => $router->buildLink('link-directory/upgrades/edit', $upgrade)
            ]];
        }, false);
    }
}