<?php

namespace xenmade\LD\Search\Data;

use XF\Mvc\Entity\Entity;
use XF\Search\Data\AbstractData;
use XF\Search\IndexRecord;
use XF\Search\MetadataStructure;
use XF\Search\Query\MetadataConstraint;

class Link extends AbstractData
{
	public function getEntityWith($forView = false)
	{
		$get = ['Category'];
		if ($forView)
		{
			$get[] = 'User';

			$visitor = \XF::visitor();
			$get[] = 'Category.Permissions|' . $visitor->permission_combination_id;
		}

		return $get;
	}

	public function getIndexData(Entity $entity)
	{
		/** @var \xenMade\LD\Entity\Link $entity */

		if (!$entity->Category)
		{
			return null;
		}

		$index = IndexRecord::create('ld_link', $entity->link_id, [
			'title' => $entity->title_,
			'message' => $entity->description,
			'date' => $entity->insert_date,
			'user_id' => $entity->user_id,
			'discussion_id' => $entity->link_id,
			'metadata' => $this->getMetaData($entity)
		]);

		if (!$entity->isVisible())
		{
			$index->setHidden();
		}

		if ($entity->tags)
		{
			$index->indexTags($entity->tags);
		}

		return $index;
	}

	protected function getMetaData(\xenMade\LD\Entity\Link $entity)
	{
		$metadata = [
			'category' => $entity->category_id,
			'link' => $entity->link_id
		];
		if ($entity->prefix_id)
		{
			$metadata['prefix'] = $entity->prefix_id;
		}

		return $metadata;
	}

	public function setupMetadataStructure(MetadataStructure $structure)
	{
		$structure->addField('category', MetadataStructure::INT);
		$structure->addField('link', MetadataStructure::INT);
		$structure->addField('prefix', MetadataStructure::INT);
	}

	public function getResultDate(Entity $entity)
	{
		return $entity->insert_date;
	}

	public function getTemplateData(Entity $entity, array $options = [])
	{
		return [
			'link' => $entity,
			'options' => $options
		];
	}

	public function canUseInlineModeration(Entity $entity, &$error = null)
	{
		/** @var \xenMade\LD\Entity\Link $entity */
		return $entity->canUseInlineModeration($error);
	}

	public function getSearchableContentTypes()
	{
		return ['link'];
	}

	public function getSearchFormTab()
	{
		/** @var \xenMade\LD\XF\Entity\User $visitor */
		$visitor = \XF::visitor();
		if (!method_exists($visitor, 'canViewLinks') || !$visitor->canViewLinks())
		{
			return null;
		}

		return [
			'title' => \XF::phrase('ld_search_links'),
			'order' => 300
		];
	}

	public function getSectionContext()
	{
		return 'ld_link';
	}

	public function getSearchFormData()
	{
		$prefixListData = $this->getPrefixListData();

		return [
			'prefixGroups' => $prefixListData['prefixGroups'],
			'prefixesGrouped' => $prefixListData['prefixesGrouped'],

			'categoryTree' => $this->getSearchableCategoryTree()
		];
	}

	/**
	 * @return \XF\Tree
	 */
	protected function getSearchableCategoryTree()
	{
		/** @var \xenMade\LD\Repository\Category $categoryRepo */
		$categoryRepo = \XF::repository('xenMade\LD:Category');
		return $categoryRepo->createCategoryTree($categoryRepo->getViewableCategories());
	}

	protected function getPrefixListData()
	{
		/** @var \xenMade\LD\Repository\LinkPrefix $prefixRepo */
		$prefixRepo = \XF::repository('xenMade\LD:LinkPrefix');
		return $prefixRepo->getVisiblePrefixListData();
	}

	public function applyTypeConstraintsFromInput(\XF\Search\Query\Query $query, \XF\Http\Request $request, array &$urlConstraints)
	{
		$prefixes = $request->filter('c.prefixes', 'array-uint');
		$prefixes = array_unique($prefixes);
		if ($prefixes && reset($prefixes))
		{
			$query->withMetadata('prefix', $prefixes);
		}
		else
		{
			unset($urlConstraints['prefixes']);
		}

		$categoryIds = $request->filter('c.categories', 'array-uint');
		$categoryIds = array_unique($categoryIds);
		if ($categoryIds && reset($categoryIds))
		{
			if ($request->filter('c.child_categories', 'bool'))
			{
				$categoryTree = $this->getSearchableCategoryTree();

				$searchCategoryIds = array_fill_keys($categoryIds, true);
				$categoryTree->traverse(function($id, $category) use (&$searchCategoryIds)
				{
					if (isset($searchCategoryIds[$id]) || isset($searchCategoryIds[$category->parent_category_id]))
					{
						$searchCategoryIds[$id] = true;
					}
				});

				$categoryIds = array_unique(array_keys($searchCategoryIds));
			}
			else
			{
				unset($urlConstraints['child_categories']);
			}

			$query->withMetadata('category', $categoryIds);
		}
		else
		{
			unset($urlConstraints['categories']);
			unset($urlConstraints['child_categories']);
		}

		$includeUpdates = $request->filter('c.include_updates', 'bool');
		if (!$includeUpdates)
		{
			$query->inType('ld_link');
		}
	}

	public function getTypePermissionConstraints(\XF\Search\Query\Query $query, $isOnlyType)
	{
		/** @var \xenMade\LD\Repository\Category $categoryRepo */
		$categoryRepo = \XF::repository('xenMade\LD:Category');

		$with = ['Permissions|' . \XF::visitor()->permission_combination_id];
		$categories = $categoryRepo->findCategoryList(null, $with)->fetch();

		$skip = [];
		foreach ($categories AS $category)
		{
			/** @var \xenMade\LD\Entity\Category $category */
			if (!$category->canView())
			{
				$skip[] = $category->category_id;
			}
		}

		if ($skip)
		{
			return [
				new MetadataConstraint('category', $skip, MetadataConstraint::MATCH_NONE)
			];
		}
		else
		{
			return [];
		}
	}

	public function getGroupByType()
	{
		return 'ld_link';
	}
}