<?php

namespace xenMade\LD\Service\Link;

use xenMade\LD\Entity\Category;

class Create extends \XF\Service\AbstractService
{
	use \XF\Service\ValidateAndSavableTrait;

	protected $category;

	protected $link;

	protected $tagChanger;

	protected $withOutDescription = false;

    /**
     * @var \xenMade\LD\Service\Link\Preparer
     */
	protected $preparer;

    protected $performValidations = true;

    protected $logIp = true;

	public function __construct(\XF\App $app, Category $category)
	{
		parent::__construct($app);
		$this->category = $category;
		$this->setupDefaults();
	}

	protected function setupDefaults()
	{
        $link = $this->category->getNewLink();

		$this->link = $link;

		$this->tagChanger = $this->service('XF:Tag\Changer', 'ld_link', $this->category);

        $this->preparer = $this->service('xenMade\LD:Link\Preparer', $this->link);

		$visitor = \XF::visitor();
		$this->link->user_id = $visitor->user_id;
		$this->link->username = $visitor->username;
		$this->link->state = $this->category->getNewContentState();
	}

	public function getCategory()
	{
		return $this->category;
	}

	public function getLink()
	{
		return $this->link;
	}

	public function setPerformValidations($perform)
	{
		$this->performValidations = (bool)$perform;
	}

    public function setWithOutDescription($status)
    {
        $this->withOutDescription = $status;
    }

    public function getWithOutDescription()
    {
        return $this->withOutDescription;
    }


    public function getPerformValidations()
	{
		return $this->performValidations;
	}

	public function setTitle($title)
	{
		$this->link->title = $title;
	}

    public function setDescription($description)
    {
        $this->link->description = $description;
    }

    public function setUrl($url)
    {
        $this->preparer->setUrl($url);
    }

    public function setBacklink()
    {
        $this->preparer->setBacklink();
    }

	public function setContent($title, $description)
	{
		$this->setTitle($title);
		$this->setDescription($description);
	}

	public function setPrefix($prefixId)
	{
		$this->link->prefix_id = $prefixId;
	}

	public function setTags($tags)
	{
		if ($this->tagChanger->canEdit())
		{
			$this->tagChanger->setEditableTags($tags);
		}
	}

	public function setCustomFields(array $customFields)
	{
		$link = $this->link;

		/** @var \XF\CustomField\Set $fieldSet */
		$fieldSet = $link->custom_fields;

		$fieldDefinition = $fieldSet->getDefinitionSet()
			->filterEditable($fieldSet, 'user')
			->filterOnly($this->category->field_cache);

		$customFieldsShown = array_keys($fieldDefinition->getFieldDefinitions());

		if ($customFieldsShown)
		{
			$fieldSet->bulkSet($customFields, $customFieldsShown);
		}
	}

	public function logIp($logIp)
	{
        $this->logIp = $logIp;
	}

	public function checkForSpam()
	{
		if ($this->link->state == 'visible' && \XF::visitor()->isSpamCheckRequired())
		{
            /** @var \xenMade\LD\Entity\Link $link */
            $link = $this->link;

            /** @var \XF\Entity\User $user */
            $user = $link->User ?: $this->repository('XF:User')->getGuestUser($link->username);

            $message = $link->title . "\n" . $link->description;

            $checker = $this->app->spam()->contentChecker();

            $checker->check($user, $message, [
                'permalink' => $this->app->router('public')->buildLink('canonical:link-directory', $link),
                'content_type' => 'ld_link'
            ]);

            $decision = $checker->getFinalDecision();
            switch ($decision)
            {
                case 'moderated':
                    $link->state = 'moderated';
                    break;

                case 'denied':
                    $checker->logSpamTrigger('ld_link', null);
                    $link->error(\XF::phrase('your_content_cannot_be_submitted_try_later'));
                    break;
            }
		}
	}

	protected function finalSetup()
	{
	}

	protected function _validate()
	{
		$this->finalSetup();

		/** @var \xenMade\LD\Entity\Link $link */
		$link = $this->link;

		if($this->getWithOutDescription())
        {
            $link->structure()->columns['description']['required'] = false;
        }

		if (!$link->user_id)
		{
			/** @var \XF\Validator\Username $validator */
			$validator = $this->app->validator('Username');
            $link->username = $validator->coerceValue($link->username);

			if ($this->performValidations && !$validator->isValid($link->username, $error))
			{
				return [
					$validator->getPrintableErrorValue($error)
				];
			}
		}

        $link->preSave();
		$errors = $link->getErrors();


		if ($this->performValidations)
		{
			if (!$link->prefix_id
				&& $this->category->require_prefix
				&& $this->category->getUsablePrefixes()
			)
			{
				$errors[] = \XF::phraseDeferred('please_select_a_prefix');
			}

			if ($this->tagChanger->canEdit())
			{
				$tagErrors = $this->tagChanger->getErrors();
				if ($tagErrors)
				{
					$errors = array_merge($errors, $tagErrors);
				}
			}
		}

		return $errors;
	}

	protected function _save()
	{
        $link = $this->link;
        $category = $this->category;

		$db = $this->db();
		$db->beginTransaction();

        $link->save(true, false);

        if ($this->logIp)
        {
            $ip = ($this->logIp === true ? $this->app->request()->getIp() : $this->logIp);
            $this->writeIpLog($ip);
        }

		if ($this->tagChanger->canEdit())
		{
			$this->tagChanger
				->setContentId($link->link_id, true)
				->save($this->performValidations);
		}

		$db->commit();

		return $link;
	}

	public function sendNotifications()
	{
		if ($this->link->isVisible())
		{
			/** @var \xenMade\LD\Service\Link\Notify $notifier */
			$notifier = $this->service('xenMade\LD:Link\Notify', $this->link, 'add');
			$notifier->notifyAndEnqueue(3);
		}
	}

    protected function writeIpLog($ip)
    {
        /** @var \XF\Repository\IP $ipRepo */
        $ipRepo = $this->repository('XF:Ip');
        $ipEnt = $ipRepo->logIp($this->link->user_id, $ip, 'ld_link', $this->link->link_id);

        if ($ipEnt)
        {
            $this->link->fastUpdate('ip_id', $ipEnt->ip_id);
        }
    }

}